"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

import os
from aurori import app_bp
from flask import redirect, url_for, send_from_directory


# index route
@app_bp.route('/')
def index():
    return send_from_directory('../client', 'index.html')
    #return render_template("index.html")


@app_bp.route('/<path:path>')
def static_proxy(path):
    # send_static_file will guess the correct MIME type
    if (os.path.isdir("./client/" + path)):
        return send_from_directory('../client/' + path, 'index.html')
    if (os.path.isfile("./client/" + path)):
        return app_bp.send_static_file(path)
    else:
        return send_from_directory('../client/section', 'index.html')


@app_bp.errorhandler(404)
def page_not_found(e):
    return redirect(url_for('index'))


@app_bp.after_request
def add_header(r):

    r.headers[
        "Cache-Control"] = "no-cache, no-store, must-revalidate, max-age=0"
    r.headers["Pragma"] = "no-cache"
    r.headers["Expires"] = "0"
    return r



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.actions.action import Action
from aurori.common.objDict import ObjDict
from aurori.logs import logManager
from aurori.actions import webclientActions
import sys
import traceback


class RemoveViewEntryActionHandler(Action):
    def __init__(self, app, db, uri="removeViewEntry"):
        self.uri = uri
        self.app = app
        self.db = db
        # logManager.info("Instance of RemoveViewEntryActionHandler created")

    def handle(self, action, user, workspace, _actionManager):
        logManager.info("Execute removal for view entry for", action['view'])
        viewname = action['view']

        if viewname in workspace.dataViews:
            view = workspace.dataViews[viewname]
            # check if login required for this view
            if view.requireLogin is True and user is None:
                notification_action = webclientActions.NotificationAction.generate(
                    "Login required", "error")
                route_action = webclientActions.RouteAction.generate('',
                                                                     delay=3)
                return 'success', [notification_action, route_action]
            else:
                # build actions to get view
                response_actions = []
                try:
                    if view.entrykey not in action['entry']:
                        notification_action = webclientActions.NotificationAction.generate(
                            "UpdateViewEntryActionHandler miss entrykey",
                            "error")
                        response_actions = [notification_action]
                    else:
                        view.dataSyncs = []
                        view.removeViewEntryHandler(
                            user, workspace,
                            action['entry'][str(view.entrykey)])
                        self.db.session.commit()
                        for v in view.dataSyncs:
                            updateView = workspace.dataViews[v['view']]
                            dictionary = v['query']
                            entries = updateView.getViewHandler(
                                user, workspace, None
                                if dictionary is None else ObjDict(dictionary))
                            if type(entries) == tuple:
                                meta_data = entries[1]
                                entries = entries[0]
                            else:
                                meta_data = {}
                            properties = updateView.getProperties()
                            uri = workspace.uri + '/' + updateView.uri
                            loadviewaction = webclientActions.LoadViewAction.generate(
                                uri, properties, entries, meta_data)
                            response_actions.append(loadviewaction)
                    response_actions.append(
                        webclientActions.NotificationAction.generate(
                            "Removed successfully", "success"))
                    return 'success', response_actions
                except Exception as e:
                    notification_action = webclientActions.NotificationAction.generate(
                        "RemoveViewEntry '" + str(action['view']) +
                        "' failed with: " + str(e), "error")
                    response_actions = [notification_action]
                    logManager.error(
                        str(type(e).__name__) +
                        'in ExecuteViewActionsActionHandler ' + action['view'])
                    traceback.print_exc(file=sys.stdout)
                return 'success', response_actions

        notification_action = webclientActions.NotificationAction.generate(
            "View >" + viewname + "< not found", "error")
        return 'success', [notification_action]


class CreateViewEntryActionHandler(Action):
    def __init__(self, app, db, uri="createViewEntry"):
        self.uri = uri
        self.app = app
        self.db = db
        # logManager.info("Instance of CreateViewEntryActionHandler created")

    def handle(self, action, user, workspace, actionManager):
        logManager.info("Execute creation of view entry for " + action['view'])
        viewname = action['view']

        if viewname in workspace.dataViews:
            view = workspace.dataViews[viewname]
            # check if login required for this view
            if view.requireLogin is True and user is None:
                notification_action = webclientActions.NotificationAction.generate(
                    "Login required", "error")
                route_action = webclientActions.RouteAction.generate('',
                                                                     delay=3)
                return 'success', [notification_action, route_action]
            else:
                # build actions to get view
                response_actions = []
                response_data = None
                try:
                    view.dataSyncs = []
                    dictionary = action['entry']
                    response_data = view.createViewEntryHandler(
                        user, workspace, ObjDict(dictionary))
                    try:
                        notification_action = webclientActions.NotificationAction.generate(
                            response_data['notification']['message'],
                            response_data['notification']['type'],
                        )
                        response_actions.append(notification_action)
                    except Exception:
                        pass
                    self.db.session.commit()
                    for v in view.dataSyncs:
                        updateView = workspace.dataViews[v['view']]
                        dictionary = v['query']
                        entries = updateView.getViewHandler(
                            user, workspace, None)
                        if type(entries) == tuple:
                            meta_data = entries[1]
                            entries = entries[0]
                        else:
                            meta_data = {}
                        properties = updateView.getProperties()
                        uri = workspace.uri + '/' + updateView.uri
                        loadviewaction = webclientActions.LoadViewAction.generate(
                            uri, properties, entries, meta_data)
                        response_actions.append(loadviewaction)
                    if response_data is not None:
                        return 'success', response_actions, response_data
                    else:
                        return 'success', response_actions
                except Exception as e:
                    notification_action = webclientActions.NotificationAction.generate(
                        "CreateViewEntry '" + str(action['view']) +
                        "' failed with: " + str(e), "error")
                    response_actions = [notification_action]
                    logManager.error(
                        str(type(e).__name__) +
                        'in ExecuteViewActionsActionHandler ' + action['view'])
                    traceback.print_exc(file=sys.stdout)
                return 'success', response_actions

        notification_action = webclientActions.NotificationAction.generate(
            "View >" + viewname + "< not found", "error")
        return 'success', [notification_action]


class UpdateViewEntryActionHandler(Action):
    def __init__(self, app, db, uri="updateViewEntry"):
        self.uri = uri
        self.app = app
        self.db = db
        # logManager.info("Instance of UpdateViewEntryActionHandler created")

    def handle(self, action, user, workspace, actionManager):
        logManager.info("Execute update of view entry for '{}'".format(
            action['view']))
        viewname = action['view']

        if viewname in workspace.dataViews:
            view = workspace.dataViews[viewname]
            # check if login required for this view
            if view.requireLogin is True and user is None:
                notification_action = webclientActions.NotificationAction.generate(
                    "Login required", "error")
                route_action = webclientActions.RouteAction.generate('',
                                                                     delay=3)
                return 'success', [notification_action, route_action]
            else:
                # build actions to get view
                response_actions = []
                response_data = None
                try:
                    if view.entrykey not in action['entry']:
                        notification_action = webclientActions.NotificationAction.generate(
                            "UpdateViewEntryActionHandler miss entrykey",
                            "error")
                        response_actions = [notification_action]
                    else:
                        view.dataSyncs = []
                        dictionary = action['entry']
                        response_data = view.updateViewEntryHandler(
                            user, workspace,
                            action['entry'][str(view.entrykey)],
                            ObjDict(dictionary))
                        try:
                            notification_action = webclientActions.NotificationAction.generate(
                                response_data['notification']['message'],
                                response_data['notification']['type'],
                            )
                            response_actions.append(notification_action)
                        except Exception:
                            response_actions.append(
                                webclientActions.NotificationAction.generate(
                                    "Updated successfully", "success"))
                        self.db.session.commit()
                        for v in view.dataSyncs:
                            updateView = workspace.dataViews[v['view']]
                            dictionary = v['query']
                            entries = updateView.getViewHandler(
                                user, workspace, None
                                if dictionary is None else ObjDict(dictionary))
                            if type(entries) == tuple:
                                meta_data = entries[1]
                                entries = entries[0]
                            else:
                                meta_data = {}
                            properties = updateView.getProperties()
                            uri = workspace.uri + '/' + updateView.uri
                            loadviewaction = webclientActions.LoadViewAction.generate(
                                uri, properties, entries, meta_data)
                            response_actions.append(loadviewaction)
                    if response_data is not None:
                        return 'success', response_actions, response_data
                    else:
                        return 'success', response_actions
                except Exception as e:
                    notification_action = webclientActions.NotificationAction.generate(
                        "UpdateViewEntry '" + str(action['view']) +
                        "' failed with: " + str(e), "error")
                    response_actions = [notification_action]
                    logManager.error(
                        str(type(e).__name__) +
                        'in ExecuteViewActionsActionHandler ' + action['view'])
                    traceback.print_exc(file=sys.stdout)
                return 'success', response_actions

        notification_action = webclientActions.NotificationAction.generate(
            "View >" + viewname + "< not found", "error")
        return 'success', [notification_action]


class ExecuteViewActionsActionHandler(Action):
    def __init__(self, app, db, uri="executeViewAction"):
        self.uri = uri
        self.app = app
        self.db = db

    def handle(self, action, user, workspace, actionManager):
        logManager.info("Execute action '{}' on view '{}'".format(
            action['viewAction'], action['view']))
        viewname = action['view']
        if viewname in workspace.dataViews:
            print('found view', viewname, 'in', workspace.name,
                  workspace.dataViews[viewname])
            view = workspace.dataViews[viewname]

            # check if login required for this view
            if view.requireLogin is True and user is None:
                # login required but got no user identity
                notification_action = webclientActions.NotificationAction.generate(
                    "Login required", "error")
                route_action = webclientActions.RouteAction.generate('',
                                                                     delay=3)
                return 'success', [notification_action, route_action]
            else:
                # build actions to get view
                response_actions = []
                response_data = None
                try:
                    view.dataSyncs = []
                    dictionary = action
                    response_data = view.executeViewActionHandler(
                        user, workspace, ObjDict(dictionary))
                    try:
                        notification_action = webclientActions.NotificationAction.generate(
                            response_data['notification']['message'],
                            response_data['notification']['type'],
                        )
                    except Exception:
                        notification_action = webclientActions.NotificationAction.generate(
                            "Action '" + str(action['viewAction']) +
                            "' executed", "info")
                    response_actions.append(notification_action)
                    self.db.session.commit()
                    for v in view.dataSyncs:
                        updateView = workspace.dataViews[v['view']]
                        dictionary = v['query']
                        entries = updateView.getViewHandler(
                            user, workspace, None
                            if dictionary is None else ObjDict(dictionary))
                        if type(entries) == tuple:
                            meta_data = entries[1]
                            entries = entries[0]
                        else:
                            meta_data = {}
                        properties = updateView.getProperties()
                        uri = workspace.uri + '/' + updateView.uri
                        loadviewaction = webclientActions.LoadViewAction.generate(
                            uri, properties, entries, meta_data)
                        response_actions.append(loadviewaction)

                except Exception as e:
                    notification_action = webclientActions.NotificationAction.generate(
                        "Action '" + str(action['viewAction']) +
                        "' failed with: ", "error")
                    response_actions = [notification_action]
                    logManager.error(
                        str(type(e).__name__) +
                        'in ExecuteViewActionsActionHandler', action['view'])
                    traceback.print_exc(file=sys.stdout)
                if response_data is not None:
                    return 'success', response_actions, response_data
                else:
                    return 'success', response_actions

        # view not found
        notification_action = webclientActions.NotificationAction.generate(
            "View >" + viewname + "< not found", "error")
        return 'success', [notification_action]


class GetViewActionHandler(Action):
    def __init__(self, app, db, uri="getView"):
        self.uri = uri
        self.app = app
        self.db = db
        # logManager.info("Instance of GetViewActionHandler created")

    def handle(self, action, user, workspace, actionManager):
        logManager.info("Execute get view action for %s", action['view'])

        viewname = action['view']
        print(workspace.dataViews)
        if viewname in workspace.dataViews:
            print('found view', viewname, 'in', workspace.name,
                  workspace.dataViews[viewname])
            view = workspace.dataViews[viewname]

            # check if login required for this view
            if view.requireLogin is True and user is None:
                # login required but got no user identity
                notification_action = webclientActions.NotificationAction.generate(
                    "Login required", "error")
                route_action = webclientActions.RouteAction.generate('',
                                                                     delay=3)
                return 'success', [notification_action, route_action]
            else:
                # build actions to get view
                # this shows a notification on every view load, is this really necessary?
                # notification_action = webclientActions.NotificationAction.generate("Load view", "info")
                notification_action = []
                dictionary = action['query']
                entries = view.getViewHandler(user, workspace,
                                              ObjDict(dictionary))
                if type(entries) == tuple:
                    meta_data = entries[1]
                    entries = entries[0]
                else:
                    meta_data = {}
                properties = view.getProperties()
                uri = workspace.uri + '/' + view.uri
                loadviewaction = webclientActions.LoadViewAction.generate(
                    uri, properties, entries, meta_data)
                return 'success', [loadviewaction, notification_action]

        # view not found
        notification_action = webclientActions.NotificationAction.generate(
            "View >" + viewname + "< not found", "error")
        return 'success', [notification_action]



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.jobs.job import Job
from datetime import datetime, timedelta
from aurori.monits.types import MonitType


def update_monitor_view_item_from_event(view_item,
                                        event,
                                        floating_average_size,
                                        update_total=True):
    view_item.values_last = event.value
    if view_item.values_max is None:
        view_item.values_max = event.value
    if event.value > view_item.values_max:
        view_item.values_max = event.value
    if view_item.values_min is None:
        view_item.values_min = event.value
    if event.value < view_item.values_min:
        view_item.values_min = event.value
    if view_item.values_count is None:
        view_item.values_count = 0
    view_item.values_count += 1
    if update_total is True and view_item.total_count is not None:
        view_item.total_count += 1
    if view_item.values_sum is None:
        view_item.values_sum = 0.0
    view_item.values_sum += event.value
    if update_total is True and view_item.total_sum is not None:
        view_item.total_sum += event.value
    if view_item.values_count <= floating_average_size:
        view_item.values_average = view_item.values_sum / view_item.values_count
    else:
        view_item.values_average = (view_item.values_average *
                                    (floating_average_size - 1) +
                                    event.value) / floating_average_size

    view_item.last_event_at = event.timestamp


def get_view_item_for_classifier_from_list(view_items, classifier):
    if view_items is None or len(view_items) == 0:
        return None
    for i in view_items:
        if i.classifier == classifier:
            return i
    return None


class ProcessMonitJob(Job):

    description = "Process a monit"

    def define_arguments(self):
        self.addDictArgument('monit',
                             label="The monit",
                             description="The monit to process")

    def run(self, **kwargs):
        from aurori.monits.models import MonitorEvent, MonitorViewItem
        monit_dict = kwargs['monit']
        monit_key = kwargs['monit_key']
        floating_average_size = monit_dict.floating_average_size
        now = datetime.utcnow()
        current_hour_threshold = now.replace(minute=0, second=0, microsecond=0)
        last_hour_threshold = now.replace(minute=0, second=0,
                                          microsecond=0) - timedelta(hours=1)

        monit_events_of_actual_and_last_hour = MonitorEvent.query.filter(
            MonitorEvent.timestamp >= last_hour_threshold,
            MonitorEvent.name == monit_dict.event.name).order_by(
                MonitorEvent.timestamp.asc()).all()
        # get the events of the actual hour
        monit_events_of_actual_hour = MonitorEvent.query.filter(
            MonitorEvent.timestamp >= current_hour_threshold,
            MonitorEvent.name == monit_dict.event.name).order_by(
                MonitorEvent.timestamp.asc()).all()
        # get the events of the last hour
        monit_events_of_last_hour = MonitorEvent.query.filter(
            MonitorEvent.timestamp >= last_hour_threshold,
            MonitorEvent.timestamp <= current_hour_threshold,
            MonitorEvent.name == monit_dict.event.name).order_by(
                MonitorEvent.timestamp.asc()).all()

        # get the monits general view items
        # the gernal view items store the general statistics
        # they are stored in addition to the time aggreated view items
        monit_global_view_items = MonitorViewItem.query.filter(
            MonitorViewItem.monit_key == monit_key,
            MonitorViewItem.timestamp is None, MonitorViewItem.year is None,
            MonitorViewItem.month is None, MonitorViewItem.day is None,
            MonitorViewItem.hour is None).all()

        if (monit_dict.monit_type == MonitType.TIMELINE):
            monit_current_hour_view_items = MonitorViewItem.query.filter(
                MonitorViewItem.monit_key == monit_key,
                MonitorViewItem.timestamp is None,
                MonitorViewItem.year == current_hour_threshold.year,
                MonitorViewItem.month == current_hour_threshold.month,
                MonitorViewItem.day == current_hour_threshold.day,
                MonitorViewItem.hour == current_hour_threshold.hour).order_by(
                    MonitorViewItem.timestamp.asc()).all()

            monit_last_hour_view_items = MonitorViewItem.query.filter(
                MonitorViewItem.monit_key == monit_key,
                MonitorViewItem.timestamp is None,
                MonitorViewItem.year == last_hour_threshold.year,
                MonitorViewItem.month == last_hour_threshold.month,
                MonitorViewItem.day == last_hour_threshold.day,
                MonitorViewItem.hour == last_hour_threshold.hour).order_by(
                    MonitorViewItem.timestamp.asc()).all()
        else:
            monit_container_view_items = MonitorViewItem.query.filter(
                MonitorViewItem.monit_key == monit_key,
                MonitorViewItem.timestamp != None).order_by(
                    MonitorViewItem.timestamp.desc()).first()
            print(monit_container_view_items)

        global_general_view_item = None
        if len(monit_global_view_items) == 0:
            # create
            global_general_view_item = MonitorViewItem(monit_key=monit_key)
            self.db.session.add(global_general_view_item)
        else:
            global_general_view_item = get_view_item_for_classifier_from_list(
                monit_global_view_items, None)

        # global_classifier_view_item = None
        # if check_for_classifier(monit_global_view_items,) is None:

        for e in monit_events_of_actual_and_last_hour:
            classifier = e.classifier
            # update the global general view item for the monit
            if (global_general_view_item.last_event_at is None) \
                    or (e.timestamp > global_general_view_item.last_event_at):
                update_monitor_view_item_from_event(global_general_view_item,
                                                    e, floating_average_size)

            if classifier is not None:
                # check for global classifier view item
                global_classifier_view_item = get_view_item_for_classifier_from_list(
                    monit_global_view_items, classifier)
                if global_classifier_view_item is None:
                    global_classifier_view_item = MonitorViewItem(
                        monit_key=monit_key, classifier=classifier)
                    monit_global_view_items.append(global_classifier_view_item)
                    self.db.session.add(global_classifier_view_item)

                if (global_classifier_view_item.last_event_at is None) \
                        or (e.timestamp > global_classifier_view_item.last_event_at):
                    update_monitor_view_item_from_event(
                        global_classifier_view_item, e, floating_average_size)

            if monit_dict.monit_type == MonitType.BUFFER:
                if (monit_container_view_items is None) \
                        or (monit_container_view_items.last_event_at is None) \
                        or (e.timestamp > monit_container_view_items.last_event_at):
                    container_view_item = MonitorViewItem(
                        monit_key=monit_key, classifier=classifier)
                    container_view_item.timestamp = e.timestamp
                    update_monitor_view_item_from_event(
                        container_view_item, e, floating_average_size)
                    self.db.session.add(container_view_item)

        if (monit_dict.monit_type == MonitType.TIMELINE):
            update_total = True
            for e in monit_events_of_last_hour:
                classifier = e.classifier

                monit_last_hour_classifier_item = get_view_item_for_classifier_from_list(
                    monit_last_hour_view_items, classifier)

                if monit_last_hour_classifier_item is None:
                    monit_last_hour_classifier_item = MonitorViewItem(
                        monit_key=monit_key, classifier=classifier)
                    monit_last_hour_classifier_item.year = last_hour_threshold.year
                    monit_last_hour_classifier_item.month = last_hour_threshold.month
                    monit_last_hour_classifier_item.day = last_hour_threshold.day
                    monit_last_hour_classifier_item.hour = last_hour_threshold.hour
                    self.db.session.add(monit_last_hour_classifier_item)
                    update_total = False

                    last_view_item = get_view_item_for_classifier_from_list(
                        monit_global_view_items, classifier)

                    # compute total values
                    if last_view_item.total_count is None:
                        monit_last_hour_classifier_item.total_count = last_view_item.values_count
                    else:
                        monit_last_hour_classifier_item.total_count = last_view_item.total_count

                    if last_view_item.total_sum is None:
                        monit_last_hour_classifier_item.total_sum = last_view_item.values_sum
                    else:
                        monit_last_hour_classifier_item.total_sum = last_view_item.total_sum
                    monit_last_hour_view_items.append(
                        monit_last_hour_classifier_item)

                if (monit_last_hour_classifier_item.last_event_at is None) \
                        or (e.timestamp > monit_last_hour_classifier_item.last_event_at):
                    update_monitor_view_item_from_event(
                        monit_last_hour_classifier_item, e,
                        floating_average_size, update_total)

            update_total = True
            for e in monit_events_of_actual_hour:
                classifier = e.classifier
                monit_current_hour_classifier_item = get_view_item_for_classifier_from_list(
                    monit_current_hour_view_items, classifier)

                if monit_current_hour_classifier_item is None:
                    monit_current_hour_classifier_item = MonitorViewItem(
                        monit_key=monit_key, classifier=classifier)
                    monit_current_hour_classifier_item.year = current_hour_threshold.year
                    monit_current_hour_classifier_item.month = current_hour_threshold.month
                    monit_current_hour_classifier_item.day = current_hour_threshold.day
                    monit_current_hour_classifier_item.hour = current_hour_threshold.hour
                    self.db.session.add(monit_current_hour_classifier_item)

                    last_view_item = get_view_item_for_classifier_from_list(
                        monit_global_view_items, classifier)

                    # compute total values
                    if last_view_item.total_count is None:
                        monit_current_hour_classifier_item.total_count = last_view_item.values_count
                    else:
                        monit_current_hour_classifier_item.total_count = last_view_item.total_count

                    if last_view_item.total_sum is None:
                        monit_current_hour_classifier_item.total_sum = last_view_item.values_sum
                    else:
                        monit_current_hour_classifier_item.total_sum = last_view_item.total_sum

                    update_total = False
                    monit_current_hour_view_items.append(
                        monit_current_hour_classifier_item)

                if (monit_current_hour_classifier_item.last_event_at is None) \
                        or (e.timestamp > monit_current_hour_classifier_item.last_event_at):
                    update_monitor_view_item_from_event(
                        monit_current_hour_classifier_item, e,
                        floating_average_size, update_total)

        print(f"ProcessMonitJob for {monit_dict.name} done")
        pass



from flask import request
import arrow
import random
import string
from aurori.actions.action import Action
from aurori.logs import logManager
from aurori.actions import webclientActions
from aurori.users import userManager
from aurori.messages import send_mail


class LostPassword(Action):
    def __init__(self, app):
        # logManager.info("ProvideMenu of type Action created")
        super().__init__(app, uri="lostPassword")

    def handle(self, action, user, workspace, actionManager):
        logManager.info("Execute lost password action")
        notification_action = webclientActions.NotificationAction.generate(
            "A email to reset the password will be send to you", "success"
        )
        ref = request.referrer.split("/")
        if ref[0] != "http:" and ref[0] != "https:":
            ref = "/".join(ref[:1])
        else:
            ref = "/".join(ref[:3])

        resetUser = userManager.getUser(action.username)
        if resetUser is not None:
            key = "".join(random.choices(string.ascii_letters + string.digits, k=96))
            resetUser.password_reset_expired_date = arrow.utcnow().shift(hours=2)
            resetUser.password_reset_hash = key
            link = ref + "/user/resetpassword" + "?key=" + key
            data = {
                "username": resetUser.firstname + " " + resetUser.lastname,
                "reset_link": link,
            }
            print(key)
            send_mail(
                [resetUser.email],
                "Reset your password",
                workspace,
                "lostPassword.mail",
                data,
            )

        return "success", [notification_action]



from aurori.workspaces import DataView, Workspace

from workspaces.Users.models import User
""" A view contaning a list of all users not locked
"""


class UserList(DataView):

    uri = 'userList'
    requireLogin = True

    def defineMetadata(self):
        self.addStringMeta("test")

    def defineProperties(self):
        self.addMailProperty(name='email', label='eMail', isKey=True)
        self.addIntegerProperty(name='id', label='ID')
        self.addStringProperty(name='name', label='Name')
        self.addStringProperty(name='organization', label='Organization')
        self.addIntegerProperty(name='orderlimit', label='Order limit')
        self.addSelectProperty(name='ldap',
                               selectables=lambda: ['Yes', 'No'],
                               label='LDAP')
        self.addActionProperty(name='lock',
                               label='Lock user',
                               action='lock',
                               actionHandler=self.lockHandler,
                               icon='clear')
        self.addActionProperty(name='setAdmin',
                               action='setAdmin',
                               actionHandler=self.setAdmin,
                               label='Give admin privileges',
                               icon='flash_on',
                               color="green")
        self.addActionProperty(name='unsetAdmin',
                               actionHandler=self.unsetAdmin,
                               label='Remove admin privileges',
                               action='unsetAdmin',
                               icon='flash_off')

    def getViewHandler(self, user: User, _workspace: Workspace, _query=None):
        print("getDataViewHandler for UserView")
        userlist = []
        all_user = User.query.filter_by(account_locked=False).all()
        for u in all_user:

            assert isinstance(u, User)
            # get new empty entry
            entry = self.createEntry()

            # fill entry
            entry.id = u.id
            entry.email = u.email
            entry.name = "{0} {1}".format(u.firstname, u.lastname)
            entry.organization = u.organization
            entry.orderlimit = u.orderlimit
            entry.ldap = 'Yes' if u.ldap else 'No'

            # set entry actions
            entry.lock = False

            if user.admin is True and u.admin is False:
                entry.lock = True
                entry.setAdmin = True
                entry.unsetAdmin = False

            if user.admin is True and u.admin is True:
                entry.setAdmin = False
                entry.unsetAdmin = True

            userlist.append(entry.extract())
        userlist = sorted(userlist, key=lambda item: item['name'])
        return userlist

    # Handler for a request to update a single view entry
    def updateViewEntryHandler(self, user, _workspace, key, entry):
        user = User.query.filter_by(email=key).first()
        user.orderlimit = entry.orderlimit

        self.emitSyncUpdate(key)
        print("Handle updateViewEntryHandler request for User order limit")

    def lockHandler(self, _user, _workspace, _action, entrykey):
        locked_user = User.query.filter_by(email=entrykey).first()
        locked_user.account_locked = True
        self.emitSyncCreate(entrykey, "userLockedList")
        self.emitSyncRemove(entrykey, "userList")
        return {"a": 1}

    def __repr__(self):
        return '<{} with {} properties>'.format(self.name,
                                                len(self.properties))

    # Handler for a request to create a new view entry
    def createViewEntryHandler(self, user, workspace):
        print("Handle createViewEntry request for UserListView")

    # give user admin priviliges
    def setAdmin(self, user, _workspace, _action, key):
        user = User.query.filter_by(email=key).first()
        user.admin = True
        self.emitSyncUpdate(key, "userList")

    # remove user admin priviliges
    def unsetAdmin(self, user, _workspace, _action, key):
        user = User.query.filter_by(email=key).first()
        user.admin = False
        self.emitSyncUpdate(key, "userList")



from flask import request

from aurori.actions.action import Action
from aurori.actions import generateActionLink
from aurori.logs import logManager
from aurori.actions import webclientActions
from aurori.users import userManager
from aurori.messages import send_mail


class ResendVerificationRequest(Action):
    def __init__(self, app):
        super().__init__(app, uri="resendVerificationMail")

    def handle(self, action, user, workspace, actionManager):
        logManager.info("Execute resend activation mail action")
        notification_action = webclientActions.NotificationAction.generate(
            "The verification of your account got requested.", "success"
        )
        ref = request.referrer.split("/")
        if ref[0] != "http:" and ref[0] != "https:":
            ref = "/".join(ref[:1])
        else:
            ref = "/".join(ref[:3])

        verifyUser = userManager.getUser(action.username)
        if verifyUser is not None:
            link = generateActionLink(
                workspace,
                "verifyUser",
                {"email": verifyUser.email},
                "user/login",
                True,
                False,
            )
            data = {
                "username": verifyUser.firstname + " " + verifyUser.lastname,
                "action_link": link,
            }
            print(link)
            send_mail(
                [verifyUser.email],
                "Verify your account",
                workspace,
                "requestVerification.mail",
                data,
            )
        return "success", [notification_action]



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.logs import logManager
from aurori.jobs import jobManager, add_dated_job
import re

from aurori.openproject.jobs import CreateTicketJob


class OpenprojectManager(object):
    """ The OpenprojectManager
    """
    def __init__(self):
        # preparation to instanciate
        self.config = None
        self.app = None
        self.db = None
        self.workspaceManager = None

    def init_manager(self, app, db, workspaceManager, config):
        self.config = config
        self.app = app
        self.db = db
        self.workspaceManager = workspaceManager

        from aurori.openproject.models import OpenProject
        self.openproject = OpenProject

        jobManager.register_job(None, CreateTicketJob, True)

        if 'openproject_api_key' in config['SYSTEM']:
            key_matcher = re.compile('^([0-9a-z]{50,})$')
            if not key_matcher.match(config['SYSTEM']['openproject_api_key']):
                logManager.error(
                    "OpenprojectManager: 'openproject_api_key' doesn't look valid"
                )
        else:
            logManager.error(
                "OpenprojectManager attribute 'openproject_api_key' not defined in config under 'SYSTEM'"
            )

        logManager.info("OpenprojectManager initialized")

    def add_create_ticket_job(self, project, subject, creating_object):
        print('\n\n\n\n')
        print('{} - {} - {}'.format(project, subject, creating_object))
        args = {
            'project': project,
            'subject': subject,
            'object': creating_object,
        }
        add_dated_job(None, CreateTicketJob(), args)



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori import db
from aurori.users.basemodels import UserBase

association_table_user_permissiongroup = db.Table(
    'permissiongroup_user_map',
    db.Column('user_id', db.Integer, db.ForeignKey('users.id')),
    db.Column('permissiongroup_id', db.Integer,
              db.ForeignKey('permissiongroups.id')))


class PermissionGroup(db.Model):
    __tablename__ = 'permissiongroups'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(64), default="")
    ldap_group = db.Column(db.String(64), default="")
    description = db.Column(db.String(128), default="")
    ldap = db.Column(db.Boolean, default=False)
    users = db.relationship(
        "User",
        backref="permission_groups",
        secondary=association_table_user_permissiongroup,
        lazy='subquery',
    )

    def __repr__(self):
        return '<PermissionGroup "{}">'.format(self.name)


#   def __init__(self, name):
#       self.name = name'

association_table_permission_permissiongroup = db.Table(
    'permission_permissiongroup_map',
    db.Column('permissiongroup_id', db.Integer,
              db.ForeignKey('permissiongroups.id')),
    db.Column('permission_id', db.Integer, db.ForeignKey('permissions.id')))


class Permission(db.Model):
    __tablename__ = 'permissions'
    id = db.Column(db.Integer, primary_key=True)
    groups = db.relationship(
        "PermissionGroup",
        backref="permissions",
        secondary=association_table_permission_permissiongroup,
        lazy='subquery',
    )
    name = db.Column(db.String(64), default="")
    description = db.Column(db.String(128), default="")

    def __repr__(self):
        return '<Permission "{}">'.format(self.name)



from datetime import datetime
from aurori import db, bcrypt
"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from sqlalchemy_utils import ArrowType
import arrow

from aurori.common.jsonDict import JsonDict


class JobExecute(db.Model):
    __tablename__ = 'jobs'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(120), default="")
    workspace = db.Column(db.String(120), default="")
    job = db.Column(db.String(120), default="")
    state = db.Column(db.String(120), default="")
    triggered_on = db.Column(ArrowType, default=arrow.utcnow)
    triggered_by = db.Column(db.String(120), default="")
    lifetime = db.Column(db.Integer, default=0)
    args = db.Column(JsonDict)
    results = db.Column(JsonDict)

    def __repr__(self):
        return '<Job {} for {}/{} >'.format(self.name, self.workspace,
                                            self.job)



import requests

# make login request
jwt = {
    'actions': [{
        'action': 'login',
        'password': 'test',
        'username': 'test@fabba.space',
        'version': 1,
        'workspace': 'users'
    }],
    'data':
    None,
    'head': {
        'msgId': 1,
        'requestType': '',
        'session': 'bK0GIzkTfybtIjWPZGWt_x05f0HHB2_0sfcJYEshvLE',
        'source': 'webclient',
        'target': 'webserver',
        'version': '1.0'
    }
}
login_request = requests.post('http://127.0.0.1:5000/api/v1', json=jwt)
print(login_request.status_code, login_request.reason, login_request.content)
print('----')
print(login_request.cookies)
print("csrf_access_token: ", login_request.cookies['csrf_access_token'])

# extract cookies from login_request
cookies = requests.utils.dict_from_cookiejar(login_request.cookies)

action = {'username': 'test', 'password': 'test'}
headers = {'X-CSRF-TOKEN': login_request.cookies['csrf_access_token']}

print("")
print("make acction request with without cookies and without token:")
action_request = requests.post('http://127.0.0.1:5000/api/action')
print(action_request.status_code, action_request.reason,
      action_request.content)

print("")
print("make acction request with cookies and without token:")
action_request = requests.post('http://127.0.0.1:5000/api/action',
                               headers=headers)
print(action_request.status_code, action_request.reason,
      action_request.content)

print("")
print("make acction request with token and without cookies:")
action_request = requests.post('http://127.0.0.1:5000/api/action',
                               cookies=cookies)
print(action_request.status_code, action_request.reason,
      action_request.content)

print("")
print("make acction request with cookies and token:")
action_request = requests.post('http://127.0.0.1:5000/api/action',
                               headers=headers,
                               cookies=cookies)
print(action_request.status_code, action_request.reason,
      action_request.content)



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.logs import logManager
from datetime import datetime
from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.jobstores.sqlalchemy import SQLAlchemyJobStore
from apscheduler.events import EVENT_ALL, SchedulerEvent, JobEvent, JobExecutionEvent, JobSubmissionEvent
from cron_descriptor import ExpressionDescriptor, Options, CasingTypeEnum
from aurori.common.objDict import ObjDict
from pytz import utc
import os


class JobManager(object):
    """ The JobManager ...
    """
    def __init__(self, ):
        # preparation to instanciate
        self.config = None
        self.app = None
        self.db = None
        self.workspaceManager = None
        self.job_counter = 0
        self.jobs = {}

    def listener(self, event):
        if type(event) is SchedulerEvent:
            logManager.debug("Got SchedulerEvent: {}".format(str(event)))
        elif type(event) is JobEvent:
            logManager.debug("Got JobEvent: {}".format(str(event)))
        elif type(event) is JobExecutionEvent:
            logManager.debug("Got JobExecutionEvent: {}".format(str(event)))
        elif type(event) is JobSubmissionEvent:
            logManager.debug("Got JobSubmissionEvent: {}".format(str(event)))
        else:
            logManager.warning("Unknown JobEvent")

    def init_manager(self, app, db, config):
        self.config = config
        self.app = app
        self.db = db
        self.scheduler = BackgroundScheduler({'apscheduler.timezone': 'UTC'})
        self.scheduler.add_listener(self.listener, EVENT_ALL)
        jobstore_db = os.environ.get('DATABASE_JOBS_URL') \
            or 'sqlite:///' + os.path.join(os.getcwd(), 'jobs.db')
        self.jobstore = SQLAlchemyJobStore(url=jobstore_db)
        self.scheduler.add_jobstore(self.jobstore, alias='sqlalchemy')
        self.scheduler.start()

        from aurori.jobs.models import JobExecute
        self.job = JobExecute

        logManager.info("MessageManager initialized")

    def get_jobs(self):
        return self.jobs

    def get_job_by_class(self, jobclass):
        for id, j in self.jobs.items():
            if jobclass == j['job_class']:
                return id
        return None

    def register_job(self, workspace, job_class, log_in_db=False):
        jobkey = ""
        workspace_name = None
        if workspace is not None:
            if type(workspace) is str:
                jobkey += workspace + '/'
                workspace_name = workspace.name
            else:
                jobkey += workspace.name + '/'
                workspace_name = workspace.name

        jobInstance = job_class()

        jobkey += jobInstance.name
        jobInstance.job_key = jobkey
        if workspace is not None:
            jobInstance.workspace = workspace.name
        job = {
            'job_class': job_class,
            'name': jobInstance.name,
            'workspace': workspace_name,
            'description': jobInstance.description,
            'parameters': jobInstance.parameters,
            'trigger': 'Internal',
            'log_in_db': log_in_db,
            'cron': jobInstance.cron,
            'day': jobInstance.day,
            'week': jobInstance.week,
            'day_of_week': jobInstance.day_of_week,
            'hour': jobInstance.hour,
            'minute': jobInstance.minute,
            'second': jobInstance.second,
        }

        if jobInstance.cron is True:
            cron_list = []

            if job_class.minute is None:
                cron_list.append("*")
            else:
                cron_list.append(job_class.minute)

            if job_class.hour is None:
                cron_list.append("*")
            else:
                cron_list.append(job_class.hour)

            if job_class.day is None:
                cron_list.append("*")
            else:
                cron_list.append(job_class.day)

            cron_list.append("*")

            if job_class.day_of_week is None:
                cron_list.append("*")
            else:
                cron_list.append(job_class.day_of_week)

            cron_string = " ".join(cron_list)
            options = Options()
            options.throw_exception_on_parse_error = False
            options.day_of_week_start_index_zero = True
            options.use_24hour_time_format = True
            options.casing_type = CasingTypeEnum.LowerCase
            descripter = ExpressionDescriptor(cron_string, options)
            logManager.info("Register repetitive job '{}' triggered {}".format(
                jobkey, descripter.get_description()))
            self.scheduler.add_job(
                jobInstance.start_job,
                kwargs=({
                    "job_id": str(jobkey)
                }),
                id=(str(jobkey)),
                trigger='cron',
                replace_existing=True,
                day=job_class.day,
                day_of_week=job_class.day_of_week,
                week=job_class.week,
                hour=job_class.hour,
                minute=job_class.minute,
                second=job_class.second,
            )
            job['trigger'] = descripter.get_description()

        self.jobs[str(jobkey)] = ObjDict(job.copy())

    def run_job(self,
                user,
                jobkey,
                args,
                date,
                max_instances=10,
                log_trigger=False):
        if jobkey in self.jobs:
            je = None
            if log_trigger is True:
                from aurori.jobs.models import JobExecute
                je = JobExecute()
                je.triggered_on = str(datetime.now())
                if user is None:
                    je.triggered_by = ""
                else:
                    je.triggered_by = user.email
                je.name = jobkey
                je.workspace = self.jobs[jobkey].workspace
                je.state = "TRIGGERED"
                self.db.session.add(je)
                self.db.session.commit()

            # if self.jobs[jobkey]['cron']:
            #     # handle a cron job
            #     job = self.scheduler.get_job(jobkey)
            #     job.modify(next_run_time=datetime.now())
            #     return None
            # else:

            # handle a single trigger job
            jobInstance = self.jobs[jobkey]['job_class']()
            self.job_counter += 1
            job_ececution_id = None

            if je is not None:
                job_ececution_id = je.id

            kwargs = {
                "job_id": str(jobkey) + str(self.job_counter),
                "job_execution_id": job_ececution_id
            }
            kwargs = {**kwargs, **args}
            self.scheduler.add_job(
                jobInstance.start_job,
                id=(str(jobkey) + str(self.job_counter)),
                trigger='date',
                next_run_time=date.astimezone(utc),
                kwargs=kwargs,
                max_instances=max_instances,
            )
            if je is not None:
                return je.id
            else:
                return None

        else:
            logManager.error("Unknown type of job in add_dated_job")
            return None



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from enum import Enum
from aurori.logs import logManager


class NodeStatus(Enum):
    UNKNOWN = "UNKNOWN"


class NodeClass(object):
    """Base class that each node class have to inherit from. 
       The class define methods that all nodes must implement
    """

    disable = False  # enable or disable detection
    name = None  # overwrites the name (default = self.__class__.__name__)
    identifier = "UNKNOWN"  # node identifier to map hardware nodes with the node class
    description = "UNKNOWN"
    version = "1.0"  # version of the node this class should handle

    def __init__(self, identifier=identifier, name=name):
        self.description = 'UNKNOWN'
        if name is None:
            self.name = self.__class__.__name__
        self.status = NodeStatus.UNKNOWN

    def init_node(self, app, db):
        self.app = app
        self.db = db


"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

import smtplib
from email.mime.text import MIMEText
from jinja2 import Template
from aurori.jobs.job import Job
import ssl


class MailFromFileTemplateJob(Job):

    description = "Send a mail from a local template file with data"

    def defineArguments(self):
        self.addListArgument('recipients',
                             label="Recipients",
                             description="List of recipients email adresses")
        self.addStringArgument('subject',
                               label="Subject",
                               description="Subject of the mail")
        self.addStringArgument('template_path',
                               label="Template path",
                               description="File path to the template")
        self.addDictArgument('data',
                             label="Template data",
                             description="Data for the template")

    def run(self, **kwargs):
        config = kwargs['mail_config']
        subject = kwargs['subject']
        recipients = kwargs['recipients']
        template_path = kwargs['template_path']
        data = kwargs['data']

        sender = config.get('sender')

        # Get File Content in String
        jinja2_template_string = open(template_path, 'r').read()

        # Create Template Object
        template = Template(jinja2_template_string)

        # Render HTML Template String
        content = template.render(**data)

        # try to send message
        try:
            msg = MIMEText(content)
            msg['Subject'] = subject
            msg['From'] = sender
            msg['To'] = ','.join(recipients)

            host = config.get('server')
            port = config.get('port')
            username = config.get('username')
            password = config.get('password')
            try:
                s = smtplib.SMTP_SSL(host, port)
            except Exception:
                context = ssl.SSLContext(ssl.PROTOCOL_TLS)
                s = smtplib.SMTP(host, port)
                s.starttls(context=context)
            s.login(username, password)

            #print(sender, recipients, host, port, username, password)
            s.send_message(msg)
            s.quit()
        except Exception as e:
            print(e)
            raise e



from aurori.files import files_bp
from flask import request, jsonify, Response
from flask_jwt_extended import jwt_required, get_jwt_identity
from aurori.logs import logManager
from aurori.users import userManager

from . import fileManager


@files_bp.route('/api/v1/file/download/<uuid_string>', methods=['GET'])
@jwt_required
def file_request(uuid_string):
    jwt_id = get_jwt_identity()
    user = None
    if jwt_id:
        user = (userManager.getUser(jwt_id))

    if user:
        file, file_name, mime_type = fileManager.get_file(uuid_string)
        if file:
            logManager.info('File "{}" downloaded by user {}'.format(
                file_name, user))
            response = Response(file, mimetype=mime_type)
            response.headers.set('Content-Disposition',
                                 'attachment',
                                 filename=file_name)
            return response
        else:
            return 'File not found.', 404
    return 'Not authorized.', 401


@files_bp.route('/api/v1/file/upload', methods=['POST'])
@jwt_required
def upload_request():
    jwt_id = get_jwt_identity()
    user = None
    if jwt_id:
        user = (userManager.getUser(jwt_id))

    try:
        workspace = request.form.get('workspace')
    except Exception:
        workspace = None

    if user:
        logManager.info('File upload for user {}'.format(user))
        uploaded_files = []
        for file in request.files.values():
            success, file_info = fileManager.store_file(file,
                                                        user,
                                                        workspace=workspace)
            if success:
                uploaded_files.append(file_info)
            else:
                return 'Upload failed', 500
        return jsonify(uploaded_files), 200
    return 'Not authorized.', 401


@files_bp.route('/api/v1/file/delete/<uuid_string>', methods=['GET'])
@jwt_required
def delete_request(uuid_string):
    jwt_id = get_jwt_identity()
    user = None
    if jwt_id:
        user = (userManager.getUser(jwt_id))

    if user:
        status, file_name = fileManager.delete_file(uuid_string)
        if status == 'SUCCESS':
            logManager.info('File "{}" deleted by user {}'.format(
                file_name, user))
            return 'Successfully deleted', 200
        else:
            return 'Deletion failed', 500
    return 'Not authorized.', 401



import arrow
from aurori.actions.action import Action
from aurori.logs import logManager
from aurori.actions import webclientActions

from workspaces.Users.models import User

class ResetPassword(Action):
    def __init__(self, app):
        super().__init__(app, uri="resetPassword")

    def handle(self, action, user, workspace, actionManager):
        user = User.query.filter_by(password_reset_hash=action.resetKey).first()
        if user is None:
            notification_action = webclientActions.NotificationAction.generate(
                "Key is not valid or expired", "error"
            )
        else:
            if user.password_reset_expired_date < arrow.utcnow():
                notification_action = webclientActions.NotificationAction.generate(
                    "Key is not valid or expired", "error"
                )
            else:
                notification_action = webclientActions.NotificationAction.generate(
                    "Password changed", "success"
                )
                user.password = action.password

        logManager.info("Request password reset", user)
        return "success", [notification_action]



import datetime
import arrow
from flask_jwt_extended import create_access_token
import re
import json

from aurori.actions.action import Action
from aurori.logs import logManager
from aurori.actions import webclientActions
from aurori.monits import monit_manager

from workspaces.Users.monits import UserLoginEvent


class Login(Action):
    def __init__(self, app):
        # logManager.info("Login of type Action created")
        super().__init__(app, uri='login')

    def handle(self, action, user, workspace, actionManager):
        logManager.info("Execute login action")
        replyActions = []
        user = (actionManager.userManager.getUser(action['username']))
        if user is not None:
            if (actionManager.userManager.checkUserPassword(
                    user, action['password'])):
                menuBuilder = actionManager.menuBuilder
                # create jwt token
                expires = datetime.timedelta(days=1)
                access_token = create_access_token(identity=action['username'],
                                                   expires_delta=expires)
                # update menu
                menu = menuBuilder.buildMenu(user)
                # build up
                userinfo = {
                    'email': user.email,
                    'limit': user.orderlimit,
                    'firstname': user.firstname,
                    'lastname': user.lastname,
                }
                replyActions.append(
                    webclientActions.UpdateSessionTokenAction.generate(
                        access_token, userinfo))
                replyActions.append(
                    webclientActions.UpdateMenuAction.generate(menu))
                replyActions.append(
                    webclientActions.NotificationAction.generate(
                        "Login successful.", "success"))

                # fixme: this doesn't handle given routes, but does the same as the other case
                if 'route' in action['options']:
                    if action['options']['route'] is True:
                        replyActions.append(
                            webclientActions.RouteAction.generate(
                                "dashboard", 3))
                elif 'params' in action['options'] and 'next' in action[
                        'options']['params']:
                    # this is the working forward with fallback
                    try:
                        parameter_string = ''
                        if 'query_parameters' in action['options']['params']:
                            parameter_dict = json.loads(
                                action['options']['params']
                                ['query_parameters'])
                            parameter_list = []
                            try:
                                for param, value in parameter_dict.items():
                                    # fixme: sanitize parameters?
                                    parameter_list.append('{}={}'.format(
                                        param, value))
                            except Exception:
                                pass
                            if len(parameter_list) > 0:
                                parameter_string = '?{}'.format(
                                    '&'.join(parameter_list))
                        forward_page = '{}{}'.format(
                            re.sub('^/', '',
                                   action['options']['params']['next']),
                            parameter_string,
                        )
                        replyActions.append(
                            webclientActions.RouteAction.generate(
                                forward_page))
                    except Exception:
                        replyActions.append(
                            webclientActions.RouteAction.generate("dashboard"))
                else:
                    replyActions.append(
                        webclientActions.RouteAction.generate("dashboard"))

                user.sessionValid = True
                user.last_login_date = arrow.utcnow()
                monit_manager.raise_monitor_event(
                    UserLoginEvent(1, "successful"))
            else:
                monit_manager.raise_monitor_event(UserLoginEvent(1, "failed"))
                replyActions.append(
                    webclientActions.NotificationAction.generate(
                        "Login failed with username or password.", "error"))
        else:
            replyActions.append(
                webclientActions.NotificationAction.generate(
                    "Login failed with username or password.", "error"))

        return 'success', replyActions



from aurori import db
from sqlalchemy import and_
from datetime import datetime
import enum
import uuid
import sys
from importlib.machinery import SourceFileLoader
from urllib.parse import quote
#from werkzeug.utils import secure_filename


class FileStorageType(enum.IntFlag):
    LOCAL = 1
    NETWORK = 2


class FileStorage(db.Model):
    __tablename__ = 'files_filestorages'

    # "server_default" is set on database by migration
    id = db.Column(db.Integer, primary_key=True)
    storage_type = db.Column(db.Enum(FileStorageType),
                             nullable=False,
                             server_default='LOCAL')
    path = db.Column(db.String, default='')
    is_default = db.Column(db.Boolean, nullable=False, server_default='0')

    def __init__(self, storage_type=None, path=None, is_default=False):
        if storage_type:
            self.storage_type = storage_type
        if path:
            self.path = path
        if is_default:
            self.set_default()

    def __repr__(self):
        return '<{} of type {} with path: {} (is default: {})>'.format(
            self.id,
            self.storage_type.name,
            self.path,
            self.is_default,
        )

    def set_default(self):
        # unset others before setting self
        storages = FileStorage.query.filter(
            and_(
                FileStorage.is_default == True,  # NOQA: E712
                FileStorage.id != self.id,
            )).all()
        for storage in storages:
            storage.is_default = False
        self.is_default = True
        # fixme: when changing the default storage, app reload is required
        # we should update the default in the fileManager

    def store_file(self, file, user, display_name=None, workspace=None):
        try:
            file_object = File(self.id)
            # fixme: we shoudl use secure_filename(file.filename) and store the original name in display_name
            file_object.file_name = file.filename
            file_object.user_id = user.id
            file_object.size = sys.getsizeof(file)
            if display_name:
                file_object.display_name = display_name
            if workspace:
                file_object.workspace = workspace

            backend_name = self.storage_type.name.lower()
            storage_backend = SourceFileLoader(
                'storage_backend',
                f'core/files/storage_backends/{backend_name}.py').load_module(
                )
            storage = storage_backend.Storage()
            if storage.store_file(self, file, file_object) is not True:
                return False, {}

            db.session.add(file_object)
            db.session.commit()

            return True, {
                'id': file_object.id,
                'file_name': file_object.file_name,
            }
        except Exception:
            return False, {}

    def load_file(self, file_object):
        try:
            backend_name = self.storage_type.name.lower()
            storage_backend = SourceFileLoader(
                'storage_backend',
                f'core/files/storage_backends/{backend_name}.py').load_module(
                )
            storage = storage_backend.Storage()
            file_data = storage.load_file(file_object)
            return file_data
        except Exception:
            return None

    def delete_file(self, file_object):
        try:
            backend_name = self.storage_type.name.lower()
            storage_backend = SourceFileLoader(
                'storage_backend',
                f'core/files/storage_backends/{backend_name}.py').load_module(
                )
            storage = storage_backend.Storage()
            storage.remove_file(file_object)
            db.session.delete(file_object)
            db.session.commit()
            return True
        except Exception:
            return False


class File(db.Model):
    __tablename__ = 'files_files'

    id = db.Column(db.Integer, primary_key=True)
    file_name = db.Column(db.String, nullable=False)
    display_name = db.Column(db.String, default='')
    workspace = db.Column(db.String, default='')
    uuid_string = db.Column(db.String)
    date_added = db.Column(db.DateTime, nullable=False)
    size_in_bytes = db.Column(db.Integer, default=0)
    filestorage_id = db.Column(db.Integer,
                               db.ForeignKey(FileStorage.id),
                               nullable=False)
    filestorage = db.relationship(
        FileStorage,
        backref='files',
        foreign_keys=filestorage_id,
    )
    user_id = db.Column(db.Integer,
                        db.ForeignKey('users.id', ondelete='SET NULL'))
    user = db.relationship(
        'User',
        backref='files',
        foreign_keys=user_id,
    )
    path = db.Column(db.String)
    mime_type = db.Column(db.String)

    def __init__(self, filestorage_id):
        self.filestorage_id = filestorage_id
        self.date_added = datetime.utcnow()
        self.uuid_string = str(uuid.uuid4())

    def __repr__(self):
        return 'File {} on file storage [{}], added {} by user with id [{}]'.format(
            self.file_name,
            self.filestorage_id,
            self.date_added.strftime('%Y-%m-%d %H:%M:%s'),
            self.user_id,
        )

    @property
    def file_data(self):
        return self.filestorage.load_file(self)

    @property
    def link(self):
        # this kind of links has to be handled by the webserver
        # fixme: this works for local files only
        if self.filestorage.storage_type.name == 'LOCAL':
            link = '/uploads/{}/{}/{}/{}/{}'.format(
                self.workspace or 'generic',
                self.uuid_string[0:2],
                self.uuid_string[2:4],
                self.uuid_string,
                self.file_name,
            )
            return quote(link)
        else:
            return False

    def delete(self):
        return self.filestorage.delete_file(self)



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

import os
from flask import Blueprint
from aurori.messages.messageManager import MessageManager

messages_bp = Blueprint('messages', __name__)

messageManager = MessageManager()


def send_mail(recipients, subject, workspace, mail_template, data):
    template_path = mail_template
    if workspace is not None:
        template_path = os.path.join(workspace.path, 'templates',
                                     mail_template)
    messageManager.add_mail_job(recipients, subject, template_path, data)


def send_message(recipient_user,
                 subject,
                 workspace,
                 message_template,
                 data,
                 sender="System",
                 mail=False,
                 mail_template=None):
    if workspace is not None:
        template_path = os.path.join(workspace.path, 'templates',
                                     message_template)

    mail_template_path = mail_template
    if mail is True:
        if workspace is not None and mail_template_path is not None:
            mail_template_path = os.path.join(workspace.path, 'templates',
                                              mail_template)

    messageManager.add_message(recipient_user, subject, template_path, data,
                               sender, mail, mail_template_path)


from aurori.messages import routes



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from flask import request, make_response
from flask_jwt_extended import jwt_optional, get_jwt_identity, get_raw_jwt
from pprint import pformat
import json
import datetime

from aurori import actionManager
from aurori.api import api_bp
from aurori.logs import logManager

# this module routes the action based api on e.g. .../api/v1
# the api needs the generalized action based json-rpc form
# every app module can hold additional entrypoint in a rest based form
# e.g. '/login/refresh'


@api_bp.route('/api/v1', methods=["POST", "GET"])
@jwt_optional
def api_v1():
    logManager.debug("Call on api/v1")
    logManager.debug(pformat(request.json, depth=2, indent=2))

    a = get_raw_jwt()
    expire_date = None
    if 'exp' in a:
        expire_date = datetime.datetime.fromtimestamp(a['exp'])
    reply = actionManager.handleActionRequest(get_jwt_identity(), expire_date,
                                              request.json)
    logManager.debug("Send reply")
    logManager.debug(pformat(request.json, depth=2, indent=2))
    reply = json.dumps(reply)
    reply = make_response(reply, 200)
    return reply


@api_bp.route('/api/v1/workspaces/<workspace_id>/view/<view_id>',
              methods=['GET'])
@jwt_optional
def api_v1_get_view(workspace_id, view_id):
    pass


@api_bp.route(
    '/api/v1/workspaces/<workspace_id>/view/<view_id>/entry/<entry_id>',
    methods=['GET'])
@jwt_optional
def api_v1_get_view_entry(workspace_id, view_id, entry_id):
    pass


@api_bp.route(
    '/api/v1/workspaces/<workspace_id>/view/<view_id>/entry/<entry_id>',
    methods=['UPDATE'])
@jwt_optional
def api_v1_update_view_entry(workspace_id, view_id, entry_id):
    pass


@api_bp.route(
    '/api/v1/workspaces/<workspace_id>/view/<view_id>/entry/<entry_id>',
    methods=['DELETE'])
@jwt_optional
def api_v1_remove_view_entry(workspace_id, view_id, entry_id):
    pass


@api_bp.route(
    '/api/v1/workspaces/<workspace_id>/view/<view_id>/entry/<entry_id>',
    methods=['POST', 'PUT'])
@jwt_optional
def api_v1_add_view_entry(workspace_id, view_id, entry_id):
    pass


@api_bp.route(
    '/api/v1/workspaces/<workspace_id>/view/<view_id>/request/<view_action_id>',
    methods=['POST'])
@jwt_optional
def api_v1_request_view_action(workspace_id, view_id, view_action_id):
    pass


@api_bp.route('/api/v1/workspaces/<workspace_id>/request/<action_id>',
              methods=['POST'])
@jwt_optional
def api_v1_request_action(workspace_id, action_id):
    pass



from aurori.actions.action import Action
from aurori.logs import logManager
from aurori.actions import webclientActions

from aurori.actions import executeActionLink
from aurori.exceptions import NotFoundError, ExpiredError


class RunActionlink(Action):
    def __init__(self, app):
        # logManager.info("ProvideMenu of type Action created")
        super().__init__(app, uri="runActionlink")

    def handle(self, action, user, _workspace, _actionManager):
        logManager.info("Execute actionlink")
        response_actions = []
        try:
            response_actions.append(
                webclientActions.UpdateActionlinkStatusAction.generate(
                    "success", "Action succeed"
                )
            )
            response_actions = response_actions + executeActionLink(action.hash, user)
        except (ExpiredError, NotFoundError):
            response_actions = [
                webclientActions.UpdateActionlinkStatusAction.generate(
                    "error", "Action not found or expired"
                )
            ]
        except Exception as exception:
            logManager.error("Execute actionlink failed: {}".format(str(exception)))
            response_actions = [
                webclientActions.UpdateActionlinkStatusAction.generate(
                    "error", "Action failed"
                )
            ]

        return "success", response_actions



from aurori.workspaces import DataView, Workspace
from aurori.users import userManager

from workspaces.Users.models import User

""" A view contaning a list of all users locked
"""


class UserLockedList(DataView):

    uri = "userLockedList"
    requireLogin = True

    def defineProperties(self):
        self.addMailProperty(name="email", label="eMail", isKey=True)
        self.addStringProperty(name="name", label="Name")
        self.addActionProperty(
            name="unlock",
            label="Unlock user",
            action="unlock",
            actionHandler=self.unlockHandler,
            color="green",
            icon="undo",
        )
        self.addActionProperty(
            name="remove",
            label="Remove user",
            action="remove",
            actionHandler=self.removeHandler,
            icon="clear",
        )

    def getViewHandler(self, user: User, workspace: Workspace, query=None):
        print("getDataViewHandler for UserView")
        userlist = []
        all_user = User.query.filter_by(account_locked=True).all()
        for u in all_user:
            assert isinstance(u, User)

            # get new empty entry
            entry = self.createEntry()

            # fill entry data
            entry.email = u.email
            entry.name = "{0} {1}".format(u.firstname, u.lastname)

            # set entry actions
            entry.remove = True
            entry.unlock = True
            userlist.append(entry.extract())
        return userlist

    def removeHandler(self, _user, _workspace, _action, entrykey):
        userManager.removeUser(entrykey)
        self.emitSyncRemove(entrykey, "userLockedList")

    def unlockHandler(self, user, workspace, action, entrykey):
        locked_user = User.query.filter_by(email=entrykey).first()
        locked_user.account_locked = False
        self.emitSyncCreate(entrykey, "userList")
        self.emitSyncRemove(entrykey, "userLockedList")

    def __repr__(self):
        return "<{} with {} properties>".format(self.name, len(self.properties))

    # Handler for a request to create a new view entry
    def createViewEntryHandler(self, user, workspace, entry):
        print("Handle createViewEntry request for " + self.uri)

    # Handler for a request to update a single view entry
    def updateViewEntryHandler(self, user, workspace, key, entry):
        print("Handle updateViewEntryHandler request for " + self.uri)



class BaseStorage:

    def store_file(self, file, file_object, file_storage=None):
        return NotImplemented

    def load_file(self, file_object):
        return NotImplemented

    def remove_file(self, file_object):
        return NotImplemented



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.logs import logManager
import hashlib
try:
    import ldap
except ImportError:
    logManager.info("Unable to import ldap")

from aurori.workspaces.workspaceHooks import WorkspaceHooks


class UserManager(object):
    """ The UserManager ...
    """
    def __init__(self, ):
        # preparation to instanciate
        pass

    def init_manager(self, app, db, workspaceManager, config):
        self.config = config
        self.app = app
        self.db = db
        self.workspaceManager = workspaceManager
        logManager.info("UserManager initialized")

        self.user = workspaceManager.user_class

    def remove_user(self, email):
        u = self.user.query.filter_by(email=email).first()
        if u is not None:
            self.workspaceManager.triggerWorkspaceHooks(WorkspaceHooks.REMOVEUSER, user=u)
            self.db.session.delete(u)
            self.db.session.commit()

    def raise_user_creation(self, user):
        self.workspaceManager.triggerWorkspaceHooks(WorkspaceHooks.CREATEUSER, user=user)                

    def raise_user_removal(self, user):
        self.workspaceManager.triggerWorkspaceHooks(WorkspaceHooks.REMOVEUSER, user=user)                

    def register_user(self, userdata, ldap=False):
        if self.checkUserExist(userdata['email']):
            return None
        else:
            u = self.user(email=userdata['email'],
                          password=userdata['password'],
                          isAdmin=False)
            u.firstname = userdata['firstname']
            u.lastname = userdata['lastname']
            u.organization = userdata['organization']
            u.account_activated = True
            u.ldap = ldap
            if ldap is True:
                u.ldap_dn = userdata['ldap_dn']
                u.ldap_username = userdata['ldap_username']
            self.workspaceManager.triggerWorkspaceHooks(WorkspaceHooks.CREATEUSER, user=u)                
            self.db.session.add(u)
            self.db.session.commit()
            return u

    def registerLdapUser(self, email):
        print("")

    def checkLdapUser(self, userid):
        server = self.config['LDAP'].get('server')
        bind_dn = self.config['LDAP'].get('bind_dn')
        bind_pw = self.config['LDAP'].get('bind_pw')
        search_dn = self.config['LDAP'].get('user_dn')
        default_organization = self.config['LDAP'].get('default_organization', "")

        attrs = [
            "givenName", "sn", "cn", "userPrincipalName", "mail",
            "sAMAccountName", "displayName"
        ]

        ldap_con = ldap.initialize(server)
        ldap_con.simple_bind_s(bind_dn, bind_pw)
        fil = "(| (mail=*" + str(userid) + \
            "*) (sAMAccountName=*" + str(userid) + "*))"
        dn, user = ldap_con.search_s(search_dn, ldap.SCOPE_SUBTREE, fil,
                                     attrs)[0]

        print(user)
        userdata = {
            "email": user['mail'][0].decode("utf-8"),
            "password": "ldap",
            "firstname": user['givenName'][0].decode("utf-8"),
            "lastname": user['sn'][0].decode("utf-8"),
            "organization": default_organization,
            'ldap_dn': dn,
            'ldap_username': user['sAMAccountName'][0].decode("utf-8")
        }
        u = self.register_user(userdata, True)
        return u

    def get_user_by_api_key(self, plaintext_api_key):
        if plaintext_api_key is not None:
            api_key_hash = hashlib.sha512(
                plaintext_api_key.encode('utf8')).hexdigest()
            return self.user.query.filter_by(api_key=api_key_hash).first()
        else:
            return None

    def getUser(self, userid):
        user = self.user.query.filter_by(email=userid).first()
        if userid is not None:
            if user is None and self.config['LDAP'].get("enable", False):
                print("Search for ldap user locally")
                user = self.user.query.filter_by(ldap_username=userid).first()
                if user is None:
                    print("Search for ldap user on server")
                    try:
                        user = self.checkLdapUser(userid)
                    except Exception:
                        user = None
        return user

    def checkUserExist(self, email):
        user = self.user.query.filter_by(email=email).first()
        if user is None:
            return False
        else:
            return True

    def checkUserPassword(self, user, password):
        if user.ldap is False:
            return user.checkPassword(password)
        else:
            server = self.config['LDAP'].get('server')
            try:
                ldap_con = ldap.initialize(server)
                ldap_con.simple_bind_s(user.ldap_dn, password)
                return True
            except Exception:
                return False



""" Workspace permissions """

from aurori.workspaces import Permission

# Define your Permissions here


class ViewLogs(Permission):
    """View logs permission"""

    description = "Allow to see the logs"



""" The environment for the ordenario app. """
from aurori.workspaces import workspaceManager
from aurori.users import userManager
from aurori.messages.models import Message
from aurori.workspaces.models import Permission, PermissionGroup
from aurori.actions.models import ActionLink
from aurori.workspaces.workspaceHooks import WorkspaceHooks  # noqa: F401, F811 pylint: disable=W0611

#from aurori.users.defaultmodels import User
from workspaces.Users.models import User

def create_environment(_app, database, clean=False):
    """ Create the environment """

    print()
    print("Create environment")
    if clean is True:
        User.query.delete()
        PermissionGroup.query.delete()
        ActionLink.query.delete()
        Message.query.delete()

    all_permissions = Permission.query.all()
    permissions_dict = {}

    print("Detected permissions:")
    for permission in all_permissions:
        permissions_dict[permission.name] = permission

    sup_pg = PermissionGroup.query.filter_by(name="Supervisor").first()
    if sup_pg is None:
        sup_pg = PermissionGroup(name="Supervisor")
        sup_pg.permissions.append(permissions_dict["Log.ViewLogs"])        
        database.session.add(sup_pg)

    if User.query.filter_by(email="admin@fabba.space").first() is None:
        print("Create admin user")
        adminuser = User(email="admin@fabba.space", password="admin", isAdmin=True)
        adminuser.firstname = "Test"
        adminuser.lastname = "Admin"
        adminuser.organization = "Fabba Team"
        adminuser.account_activated = True
        adminuser.permission_groups.append(sup_pg)
        # adding the user to the database
        database.session.add(adminuser)
        # raise a event to inform about the creation
        userManager.raise_user_creation(adminuser)        

    all_permissions = Permission.query.all()
    print(all_permissions)

    database.session.commit()

    all_user = User.query.all()
    print(all_user)



"""Initialisation

Revision ID: 00000
Revises: 
Create Date: 2022-07-28 09:55:44.452130

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '00000'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('files_filestorages',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('storage_type', sa.Enum('LOCAL', 'NETWORK', name='filestoragetype'), server_default='LOCAL', nullable=False),
    sa.Column('path', sa.String(), nullable=True),
    sa.Column('is_default', sa.Boolean(), server_default='0', nullable=False),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('node_logs',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('node_name', sa.String(length=120), nullable=True),
    sa.Column('node_class', sa.String(length=120), nullable=True),
    sa.Column('node_fingerprint', sa.String(length=120), nullable=True),
    sa.Column('node_ip', sa.String(length=120), nullable=True),
    sa.Column('node_status', sa.String(length=120), nullable=True),
    sa.Column('request_action', sa.String(length=120), nullable=True),
    sa.Column('request_reply', sa.String(length=120), nullable=True),
    sa.Column('request_status', sa.String(length=120), nullable=True),
    sa.Column('request_on', sqlalchemy_utils.types.arrow.ArrowType(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('nodes',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=64), nullable=True),
    sa.Column('fingerprint', sa.String(length=120), nullable=True),
    sa.Column('class_name', sa.String(length=120), nullable=True),
    sa.Column('workspace', sa.String(length=120), nullable=True),
    sa.Column('network_ip', sa.String(length=64), nullable=True),
    sa.Column('last_request_on', sqlalchemy_utils.types.arrow.ArrowType(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_nodes_name'), 'nodes', ['name'], unique=True)
    op.create_table('users',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('_password_hash', sa.LargeBinary(length=60), nullable=False),
    sa.Column('_pin_hash', sa.LargeBinary(length=60), nullable=True),
    sa.Column('_salt', sa.String(length=128), nullable=True),
    sa.Column('_api_key_hash', sa.String(length=256), nullable=True),
    sa.Column('email', sa.String(length=120), nullable=True),
    sa.Column('firstname', sa.String(length=64), nullable=True),
    sa.Column('lastname', sa.String(length=64), nullable=True),
    sa.Column('organization', sa.String(length=64), nullable=True),
    sa.Column('phone', sa.String(length=64), nullable=True),
    sa.Column('account_created_date', sqlalchemy_utils.types.arrow.ArrowType(), nullable=True),
    sa.Column('last_login_date', sqlalchemy_utils.types.arrow.ArrowType(), nullable=True),
    sa.Column('password_reset_expired_date', sqlalchemy_utils.types.arrow.ArrowType(), nullable=True),
    sa.Column('password_reset_hash', sa.String(length=128), nullable=True),
    sa.Column('account_verified', sa.Boolean(), nullable=True),
    sa.Column('account_locked', sa.Boolean(), nullable=True),
    sa.Column('ldap', sa.Boolean(), nullable=True),
    sa.Column('ldap_dn', sa.String(length=64), nullable=True),
    sa.Column('ldap_username', sa.String(length=64), nullable=True),
    sa.Column('admin', sa.Boolean(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_users_email'), 'users', ['email'], unique=True)
    op.create_table('files_files',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('file_name', sa.String(), nullable=False),
    sa.Column('display_name', sa.String(), nullable=True),
    sa.Column('workspace', sa.String(), nullable=True),
    sa.Column('uuid_string', sa.String(), nullable=True),
    sa.Column('date_added', sa.DateTime(), nullable=False),
    sa.Column('size_in_bytes', sa.Integer(), nullable=True),
    sa.Column('filestorage_id', sa.Integer(), nullable=False),
    sa.Column('user_id', sa.Integer(), nullable=True),
    sa.Column('path', sa.String(), nullable=True),
    sa.Column('mime_type', sa.String(), nullable=True),
    sa.ForeignKeyConstraint(['filestorage_id'], ['files_filestorages.id'], ),
    sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='SET NULL'),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('node_settings',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('node_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['node_id'], ['nodes.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('node_settings')
    op.drop_table('files_files')
    op.drop_index(op.f('ix_users_email'), table_name='users')
    op.drop_table('users')
    op.drop_index(op.f('ix_nodes_name'), table_name='nodes')
    op.drop_table('nodes')
    op.drop_table('node_logs')
    op.drop_table('files_filestorages')
    # ### end Alembic commands ###



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.logs import logManager
from aurori.common.objDict import ObjDict


class WidgetManager(object):
    """ The WidgetManager ...
    """
    def __init__(self, ):
        # preparation to instanciate
        self.widgets = {}

    def init_manager(self, app, db, userManager, workspaceManager, config):
        self.config = config
        self.app = app
        self.db = db
        self.workspaceManager = workspaceManager
        self.userManager = userManager
        logManager.info("WidgetManager initialized")

    def get_widgets(self):
        return self.widgets

    def register_widget(self, workspace, widget_class):
        widgetkey = ''
        widgetkey += workspace.name.lower() + '/'

        widgetInstance = widget_class()
        widgetkey += 'widget_{}'.format(
            widgetInstance.__class__.__name__.lower())

        widget = {
            'group':
            widgetInstance.group,
            'name':
            widgetInstance.name,
            'workspace':
            workspace.name,
            'description':
            widgetInstance.description,
            'uri':
            'widget_{}'.format(widgetInstance.__class__.__name__.lower()),
            'component_path':
            'pages/{}/widgets/{}'.format(
                workspace.name.lower(),
                widgetInstance.__class__.__name__.lower()),
            'size':
            widgetInstance.size,
            'position':
            widgetInstance.position,
        }
        if widgetInstance.required_permission:
            widget['required_permission'] = '{}.{}'.format(
                workspace.name, widgetInstance.required_permission.name)

        self.widgets[str(widgetkey)] = ObjDict(widget.copy())
        return widgetInstance

    def get_widget_by_key(self, key):
        return self.widgets.get(key)



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3"

from aurori.monits.monits import Monit
from aurori.monits.events import MonitorBaseEvent
from aurori.monits.types import MonitType, MonitTimeAggregationType, MonitPropertyType


class UserLoginEvent(MonitorBaseEvent):
    name = "UserLogin"


class UserLogins(Monit):
    event = UserLoginEvent
    monit_type = MonitType.TIMELINE
    monit_time_aggregation = MonitTimeAggregationType.HOURLY
    monit_display_property = MonitPropertyType.TOTAL_COUNT

    def define_views(self):
        self.add_timelinechart_view("total",
                                    "Total Count",
                                    MonitPropertyType.TOTAL_COUNT,
                                    fill_timeline_gaps=False)
        self.add_barchart_view("count",
                               "Count",
                               MonitPropertyType.COUNT,
                               fill_timeline_gaps=True)



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.logs import logManager
import arrow
from jinja2 import Template
from aurori.jobs import jobManager, add_dated_job
from aurori.messages.mailingJob import MailFromFileTemplateJob


class MessageManager(object):
    """ The MessageManager ...
    """
    def __init__(self, ):
        # preparation to instanciate
        self.config = None
        self.app = None
        self.db = None
        self.workspaceManager = None

    def init_manager(self, app, db, workspaceManager, config):
        self.config = config
        self.app = app
        self.db = db
        self.workspaceManager = workspaceManager

        from aurori.messages.models import Message
        self.message = Message

        jobManager.register_job(None, MailFromFileTemplateJob)

        logManager.info("MessageManager initialized")

    def add_message(self,
                    recipient_user,
                    subject,
                    message_template_path,
                    data,
                    sender,
                    mail=True,
                    mail_template_path=None):
        if mail is True and mail_template_path is not None:
            user_mail = recipient_user.email
            self.add_mail_job([user_mail], subject, mail_template_path, data)
        # Add the Message
        # Get File Content in String
        jinja2_template_string = open(message_template_path, 'r').read()

        # Create Template Object
        template = Template(jinja2_template_string)

        content = template.render(**data)

        m = self.message()
        m.message_html = content
        m.subject = subject
        m.recipient = recipient_user
        m.sender_name = sender
        m.message_send_date = arrow.utcnow()
        self.db.session.add(m)
        self.db.session.commit()

    def add_mail_job(self, recipients, subject, template_path, data):
        args = {
            'mail_config': self.config['MAIL'],
            'recipients': recipients,
            'subject': subject,
            'template_path': template_path,
            'data': data
        }
        add_dated_job(None, MailFromFileTemplateJob(), args)



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori import db
from sqlalchemy_utils import ArrowType
import arrow


class Node(db.Model):
    __tablename__ = 'nodes'
    # nonvolatile data stored in the db
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(64), index=True, unique=True)
    fingerprint = db.Column(db.String(120), default="")
    class_name = db.Column(db.String(120), default="")
    workspace = db.Column(db.String(120), default="")
    network_ip = db.Column(db.String(64), default="")
    last_request_on = db.Column(ArrowType, default=arrow.utcnow)

    def __repr__(self):
        return '<Node {} {} [{}] >'.format(self.name, self.typeid,
                                           self.fingerprint)


class NodeSettings(db.Model):
    __tablename__ = 'node_settings'
    # nonvolatile data stored in the db
    id = db.Column(db.Integer, primary_key=True)
    node_id = db.Column(db.Integer, db.ForeignKey('nodes.id'))
    node = db.relationship("Node",
                           backref=db.backref("settings", uselist=True))


class NodeLog(db.Model):
    __tablename__ = 'node_logs'
    id = db.Column(db.Integer, primary_key=True)
    node_name = db.Column(db.String(120), default="")
    node_class = db.Column(db.String(120), default="")
    node_fingerprint = db.Column(db.String(120), default="")
    node_ip = db.Column(db.String(120), default="")
    node_status = db.Column(db.String(120), default="")
    request_action = db.Column(db.String(120), default="")
    request_reply = db.Column(db.String(120), default="")
    request_status = db.Column(db.String(120), default="")
    request_on = db.Column(ArrowType, default=arrow.utcnow)



from aurori.workspaces import Page

from workspaces.Log.permissions import ViewLogs

""" The log page
"""


class Log(Page):
    title = "Log"  # Shown label of the page in the menu
    group = "Admin"  # groupname multiple pages
    icon = "description"  # icon (in typeset of material design icons)
    route = "/admin/log"  # routing
    builder = "frontend"  # page get build by the client (frontend)
    rank = 0.1  # ranks (double) the page higher values are at the top of the menu
    # groups will be ranked by the sum of the rank-values of their entries
    requireLogin = True  # login is required to view the page
    requirePermission = ViewLogs()
    requireAdmin = False  # login dont need admin privileges



from logging.config import fileConfig
import logging
from sqlalchemy import engine_from_config
from sqlalchemy import pool
import sqlalchemy_utils  # noqa: F401  pylint: disable=W0611

from alembic import context

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)
logger = logging.getLogger('alembic.env')

al_logger = logging.getLogger('alembic.runtime.migration')
al_logger.setLevel(logging.ERROR)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata
import os
import sys
from pathlib import Path

file = Path(__file__).resolve()
parent, top, root = file.parent, file.parents[4], file.parents[5]

sys.path.append(str(top))
sys.path.append(str(file.parents[2]))

logger.info(f"cwd {os.getcwd()} {top} {root} {file.parents[2]}")

# fixme: set this per workspace
import workspaces.UserAccount.models  # noqa: F401  pylint: disable=W0611

alembic_datatable_name = 'alembic_version_useraccount'

from aurori import db

target_metadata = db.Model.metadata

# fixme: this is probably fine for our use case, but in configure_app()
# we also allow setting from os.environ.get('DATABASE_URL')
config.set_main_option(
    'sqlalchemy.url', 'sqlite:///' + os.path.join(top, '..', '..', 'app.db'))

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = config.get_main_option("sqlalchemy.url")
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def include_object(obj, name, type_, reflected, compare_to):
    """
    Should you include this table or not?
    """
    if reflected is False:
        if type(obj) == type(compare_to):
            logger.info(f'No changes on {name}')
        else:
            logger.info(f'Changes found for {name}')
        return True
    return False


def render_item(type_, obj, autogen_context):
    # custom render for sqalchemy_utils ChoiceType column and params
    # May be better to use:
    # https://stackoverflow.com/questions/30132370/trouble-when-using-alembic-with-sqlalchemy-utils
    print(type(obj))
    if type_ == 'type' and type(obj).__name__ == "IntEnum":
        col_type = "sa.Integer(), default=0"
        return col_type
    if type_ == 'type' and type(obj).__name__ == "IntFlag":
        col_type = "sa.Integer(), default=0"
        return col_type

    # default rendering for other objects
    return False


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """

    # this callback is used to prevent an auto-migration from being generated
    # when there are no changes to the schema
    # reference: http://alembic.zzzcomputing.com/en/latest/cookbook.html
    def process_revision_directives(context, revision, directives):
        if getattr(config.cmd_opts, 'autogenerate', False):
            script = directives[0]
            if script.upgrade_ops.is_empty():
                directives[:] = []
                logger.info('No changes in schema detected.')

    connectable = engine_from_config(
        config.get_section(config.config_ini_section),
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=target_metadata,
            render_item=render_item,
            include_object=include_object,
            version_table=alembic_datatable_name,
            process_revision_directives=process_revision_directives,
        )

        with context.begin_transaction():
            context.run_migrations()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori import db, bcrypt
from aurori.users.basemodels import UserBase
import hashlib
from sqlalchemy.ext.hybrid import hybrid_property, hybrid_method
from sqlalchemy_utils import ArrowType
import arrow

class User(UserBase):
    def __init__(self, email, password, isAdmin=False, skip_password=False):
        self.email = email
        if skip_password is False:
            self.password = password
        self.admin = isAdmin



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori import version
from aurori.logs import logManager
from aurori.common.objDict import ObjDict
from aurori.exceptions import NotFoundError, ExpiredError
from aurori.actions import webclientActions
import arrow
import random
import datetime
import string


class ActionManager(object):
    """ The ActionManager ...
    """
    def __init__(self, ):
        # preparation to instanciate
        self.actionsMap = {}

    def mapActions(self, app):
        logManager.info("")
        logManager.info("ActionManager map actions")

        # register workspaces
        for w in self.workspaceManager.workspaces:
            self.actionsMap[str(w.uri)] = {}
            # map actions for every workspace by there uri
            for a in w.actions:
                self.actionsMap[str(w.uri)][a.uri] = a

    def mapWorkspaces(self, app):
        logManager.info("")
        logManager.info("ActionManager map workspaces")
        self.workspacesMap = {}
        for w in self.workspaceManager.workspaces:
            self.workspacesMap[str(w.uri)] = w

    def init_manager(self, app, db, userManager, menuBuilder, workspaceManager,
                     nodeManager, config):
        self.app = app
        self.db = db
        self.config = config
        self.workspaceManager = workspaceManager
        self.userManager = userManager
        self.menuBuilder = menuBuilder
        self.nodeManager = nodeManager
        self.mapWorkspaces(app)
        self.mapActions(app)

        from aurori.actions.models import ActionLink
        self.actionLink = ActionLink

        logManager.info("ActionManager initialized")

    def createActionLink(self, workspace, action_uri, action_data_dict,
                         redirect_to, once, need_login, expire_days):
        al = self.actionLink()
        al.hash = ''.join(
            random.choices(string.ascii_letters + string.digits, k=96))
        al.workspace = workspace.uri
        al.action = action_uri
        al.expire_on_date = arrow.utcnow().shift(days=expire_days)
        al.action_data_json = action_data_dict
        al.run_only_once = once

        if redirect_to == "" or redirect_to is None:
            al.redirect_to = ""
        else:
            al.redirect_to = redirect_to

        al.need_login = need_login
        self.db.session.add(al)
        base_url = self.config['SYSTEM'].get('base_url', '')
        return '/'.join([base_url, 'actionlink', al.hash])

    def executeActionLink(self, hash, user):
        response_actions = []
        al = self.actionLink.query.filter_by(hash=hash).first()
        if al is None:
            raise NotFoundError
        if al.expire_on_date < arrow.utcnow():
            raise ExpiredError

        try:
            if str(al.workspace) in self.actionsMap:
                print('action workspace found')
                if al.action in self.actionsMap[al.workspace]:
                    print('action', al.action, 'found in workspace')
                    if user is None and al.need_login is True:
                        response_actions.append(
                            webclientActions.NotificationAction.generate(
                                "Login needed you will be redirected.",
                                "success"))
                        response_actions.append(
                            webclientActions.RouteAction.generate(
                                "user/login?redirect=actionlink/" + hash, 3))
                        return response_actions
                    if al.redirect_to != "":
                        response_actions.append(
                            webclientActions.RouteAction.generate(
                                al.redirect_to, 2))
                    workspace = self.workspacesMap[al.workspace]
                    print(user, workspace)
                    param = al.action_data_json
                    state, actions = self.actionsMap[al.workspace][
                        al.action].handle(ObjDict(param), user, workspace,
                                          self)
                    if state == 'success':
                        logManager.info(
                            'Actionlink succed with {} for user: {}', actions,
                            user)
                    else:
                        logManager.error(
                            'Actionlink failed with {} for user: {}', actions,
                            user)
                        raise Exception(
                            str('Action failed with {} for user: {}', actions,
                                user))
                    response_actions = response_actions + actions
                    return response_actions
                else:
                    logManager.error('action ' + al.action + ' not found in ' +
                                     al.workspace)
                    raise Exception(
                        str('action workspace: "' + al.workspace +
                            '" not found'))
            else:
                logManager.error('action workspace: "' + al.workspace +
                                 '"not found')
                raise Exception(
                    str('action workspace: "' + al.workspace + '"not found'))

        except Exception as e:
            raise e
        finally:
            if al.run_only_once is True:
                self.db.session.delete(al)

    def createDataViewActionLink(self,
                                 entrykey,
                                 workspace,
                                 data_view_uri,
                                 action_property,
                                 action=None):
        raise NotImplementedError

    def buildActionReply(self, actions, response={}):
        reply = {}
        reply['head'] = {}
        reply['head']['version'] = version
        reply['actions'] = actions
        reply['response'] = response
        return reply

    def handleActionRequest(self, identity, expire_date, request):
        actions = request['actions']
        response_actions = []
        response_data = {}
        user = None
        # print('identity = ', identity)
        for action in actions:
            if action['workspace'] in self.actionsMap:
                #print('action workspace found')
                if action['action'] in self.actionsMap[action['workspace']]:
                    #print('action', action['action'], 'found in workspace')
                    user = (self.userManager.getUser(identity))
                    workspace = self.workspacesMap[action['workspace']]
                    handle_result = self.actionsMap[action['workspace']][
                        action['action']].handle(ObjDict(action), user,
                                                 workspace, self)
                    handle_result_len = len(handle_result)

                    if handle_result_len == 1:
                        state, actions, response = handle_result, [], {}
                    elif handle_result_len == 2:
                        state, actions, response = handle_result[
                            0], handle_result[1], {}
                    elif handle_result_len == 3:
                        state, actions, response = handle_result
                    else:
                        state, actions, response = "error", [], {}

                    self.db.session.commit()
                    if state == 'success':
                        logManager.info('Action {} succed for user: {}'.format(
                            action['action'], user))
                    else:
                        logManager.error(
                            'Action {} failed for user: {}'.format(
                                action['action'], user))

                    response_intersection = response_data.keys() & response
                    if len(response_intersection) != 0:
                        logManager.warning(
                            'Action response data for {} overrided the following properties: {}'
                            .format(action['action'], response_intersection))
                    response_data = {**response_data, **response}
                    response_actions = response_actions + actions

                else:
                    logManager.error('action ' + action['action'] +
                                     ' not found in ' + action['workspace'])
            else:
                logManager.error('action workspace: "' + action['workspace'] +
                                 ' "not found')

        if expire_date is not None and identity is not None:
            difference = expire_date - datetime.datetime.now()
            remaining_minutes = difference.seconds / 60
            session_expiration_minutes = self.config['SYSTEM'].get(
                'session_expiration_minutes', 15)
            if remaining_minutes < session_expiration_minutes * 0.5:
                access_token = self.userManager.updateAccessToken(identity)
                response_actions.append(
                    webclientActions.UpdateSessionTokenAction.generate(
                        access_token))

        return self.buildActionReply(response_actions, response_data)



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

import base64
from flask import request, g
from flask_jwt_extended import jwt_optional, get_jwt_identity
from aurori.users import userManager


def check_before_request():
    g.user = None
    jwt_id = get_jwt_identity()
    auth_header = request.headers.get('Authorization')

    if jwt_id is not None:  # check for jwt token
        user = (userManager.getUser(jwt_id))
        g.user = user
    else:  # check for header authentication basic
        if auth_header is not None:
            split_auth_header = auth_header.split()
            auth_type = split_auth_header[0]
            if auth_type == "Basic":
                decoded_auth = base64.b64decode(
                    split_auth_header[1]).decode('utf8').split(':')
                if decoded_auth[
                        0] == 'api_key':  # check for api_key authentication
                    user = userManager.get_user_by_api_key(decoded_auth[1])
                    g.user = user
                else:  # check for email and password authentication
                    user = userManager.getUser(decoded_auth[0])
                    if user is not None:
                        if userManager.checkUserPassword(
                                user, decoded_auth[1]):
                            g.user = user

    request.view_args['user'] = g.user



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.logs import logManager
from aurori.jobs import jobManager, add_dated_job

from aurori.monits.monits import Monit
from aurori.monits.events import MonitorBaseEvent
from aurori.monits.types import MonitType, MonitViewStats, MonitPropertyType, MonitChartType
from aurori.monits.processMonitJob import ProcessMonitJob
from aurori.monits.triggerMonitProcessingJob import TriggerMonitsProcessingJob
from aurori.monits.cleanupMonitorEventsJob import CleanupMonitorEventsJob
from aurori.monits.cleanupMonitorItemsJob import CleanupMonitorItemsJob

import datetime


class MonitManager(object):
    """ The MonitManager ...
    """
    def __init__(self, ):
        self.monits = {}
        self.config = None
        self.app = None
        self.db = None
        self.workspaceManager = None
        self.userManager = None

    def init_manager(self, app, db, userManager, workspaceManager, config):
        self.config = config
        self.app = app
        self.db = db
        self.workspaceManager = workspaceManager
        self.userManager = userManager

        from aurori.monits.models import MonitorEvent, MonitorViewItem
        self.MonitorEvent = MonitorEvent
        self.MonitorViewItem = MonitorViewItem

        jobManager.register_job(None, ProcessMonitJob)
        jobManager.register_job(None, TriggerMonitsProcessingJob)
        jobManager.register_job(None, CleanupMonitorEventsJob)
        jobManager.register_job(None, CleanupMonitorItemsJob)
        logManager.info("MonitManager initialized")

    def _get_key_from_monit_and_workspace_name(self, monit_name,
                                               workspace_name):
        key = monit_name
        if workspace_name is not None and workspace_name != '':
            key = workspace_name + '.' + key
        else:
            # add system monit group key
            key = '~' + '.' + key
        return key

    def _get_key_from_monit(self, monit: Monit):
        if monit.workspace is not None:
            workspace_name = monit.workspace.name
        else:
            workspace_name = None
        return self._get_key_from_monit_and_workspace_name(
            monit.name, workspace_name)

    def trigger_monit_processing(self, monit: Monit):
        key = self._get_key_from_monit(monit)
        args = {
            'monit': self.monits[self._get_key_from_monit(monit)],
            'monit_key': key,
        }
        add_dated_job(None, ProcessMonitJob(), args)

    def get_monits_list(self,
                        workspace=None,
                        group=None,
                        subgroup=None,
                        get_system_monits=False):
        return self.monits

    def get_monit_view(self, monit, view_name):
        for v in monit.views:
            if v.name == view_name:
                return v
        return None

    def get_value_from_item(self, item, last_item, _monit, view):
        if item is not None:
            if view.display_property == MonitPropertyType.COUNT:
                return item.values_count
            if view.display_property == MonitPropertyType.SUM:
                return item.values_sum
            if view.display_property == MonitPropertyType.AVERAGE:
                return item.values_average
            if view.display_property == MonitPropertyType.LAST_VALUE:
                return item.values_last
            if view.display_property == MonitPropertyType.TOTAL_COUNT:
                if item is not None and item.total_count is not None:
                    return item.total_count
                else:
                    return 0
            if view.display_property == MonitPropertyType.TOTAL_SUM:
                if item is not None and item.total_sum is not None:
                    return item.total_sum
                else:
                    return 0
        else:
            if view.display_property == MonitPropertyType.COUNT:
                return 0
            if view.display_property == MonitPropertyType.SUM:
                return 0
            if view.display_property == MonitPropertyType.AVERAGE:
                return None
            if view.display_property == MonitPropertyType.LAST_VALUE:
                return None
            if view.display_property == MonitPropertyType.TOTAL_COUNT:
                if last_item is not None and last_item.total_count is not None:
                    return last_item.total_count
                else:
                    return 0
            if view.display_property == MonitPropertyType.TOTAL_SUM:
                if last_item is not None and last_item.total_sum is not None:
                    return last_item.total_sum
                else:
                    return 0

    def get_chart_type(self, monit, view):
        if monit.monit_type == MonitType.BUFFER:
            return 'scatter'
        if monit.monit_type == MonitType.PROPERTIES_ONLY:
            return 'bar'
        if monit.monit_type == MonitType.TIMELINE:
            if view.chart_type == MonitChartType.LINE:
                return 'line'
            else:
                return 'bar'

    def get_monit_view_chart_data(self, monit_key, view_name):
        monit = self.monits[monit_key]
        view = self.get_monit_view(monit, view_name)
        chart_data = {
            'timeline_data': [],
            'series_data': [],
            'xAxesType': 'category'
        }
        classifiers = self.get_monit_view_classifier(monit, view)
        classifiers_series_index = {}
        timeline = {}
        last_classifier_item = {}

        # prepare timeline data structures

        for c in classifiers:
            classifiers_series_index[c] = len(chart_data['series_data'])
            last_classifier_item[c] = None
            if c is not None:
                chart_data['series_data'].append({
                    'label':
                    c,
                    'data': [],
                    'type':
                    self.get_chart_type(monit, view)
                })
                if view.chart_type == MonitChartType.BAR:
                    chart_data['series_data'][-1]['data'].append("0")

            else:
                chart_data['series_data'].append({
                    'label':
                    view.label,
                    'data': [],
                    'type':
                    self.get_chart_type(monit, view)
                })
                if view.chart_type == MonitChartType.BAR:
                    chart_data['series_data'][-1]['data'].append("0")

        if monit.monit_type == MonitType.BUFFER:
            chart_data['xAxesType'] = 'time'
            view_items = self.MonitorViewItem.query.filter(
                self.MonitorViewItem.monit_key == monit_key,
                self.MonitorViewItem.timestamp != None,
                self.MonitorViewItem.year == None,
                self.MonitorViewItem.month == None,
                self.MonitorViewItem.day == None,
                self.MonitorViewItem.hour == None).order_by(
                    self.MonitorViewItem.timestamp.asc()).all()
            for i in view_items:
                timestamp = i.timestamp.isoformat(" ")
                last_classifier_item[i.classifier] = i
                chart_data['series_data'][classifiers_series_index[
                    i.classifier]]['data'].append([
                        timestamp,
                        self.get_value_from_item(
                            i, last_classifier_item[i.classifier], monit, view)
                    ])
                chart_data['timeline_data'].append(timestamp)
                pass

        if monit.monit_type == MonitType.TIMELINE:
            if view.chart_type == MonitChartType.BAR:
                chart_data['timeline_data'].append("")

            # chart_data['xAxesType'] = 'time'
            view_items = self.MonitorViewItem.query.filter(
                self.MonitorViewItem.monit_key == monit_key,
                self.MonitorViewItem.timestamp is None,
                self.MonitorViewItem.year is not None,
                self.MonitorViewItem.month is not None,
                self.MonitorViewItem.day is not None, self.MonitorViewItem.hour
                is not None).order_by(self.MonitorViewItem.year.asc(),
                                      self.MonitorViewItem.month.asc(),
                                      self.MonitorViewItem.day.asc(),
                                      self.MonitorViewItem.hour.asc()).all()
            last_item = None
            last_item_datetime = None
            for i in view_items:
                actual_item_datetime = datetime.datetime(year=i.year,
                                                         month=i.month,
                                                         day=i.day,
                                                         hour=i.hour)
                time_delta = datetime.timedelta(hours=1)
                if view.fill_timeline_gaps is True and last_item is not None:
                    while last_item_datetime + time_delta <= actual_item_datetime:
                        time_string = last_item_datetime.strftime(
                            "%Y-%m-%d-%Hh")
                        if time_string not in timeline:
                            timeline[time_string] = {}
                        if view.display_property == MonitPropertyType.TOTAL_SUM or view.display_property == MonitPropertyType.TOTAL_COUNT:
                            timeline[time_string][
                                last_item.classifier] = last_item
                        else:
                            timeline[time_string][last_item.classifier] = None
                        last_item_datetime += time_delta
                if last_item is not None or actual_item_datetime == last_item_datetime:
                    time_string = actual_item_datetime.strftime("%Y-%m-%d-%Hh")
                    if time_string not in timeline:
                        timeline[time_string] = {}
                    timeline[time_string][i.classifier] = i
                last_item = i
                last_item_datetime = datetime.datetime(year=i.year,
                                                       month=i.month,
                                                       day=i.day,
                                                       hour=i.hour)
                last_item_datetime += time_delta

            for k, s in timeline.items():
                chart_data['timeline_data'].append(k)
                for c in classifiers:
                    if c in s:
                        last_classifier_item[c] = s[c]
                        chart_data['series_data'][
                            classifiers_series_index[c]]['data'].append(
                                self.get_value_from_item(
                                    s[c], last_classifier_item[c], monit,
                                    view))
                    else:
                        chart_data['series_data'][
                            classifiers_series_index[c]]['data'].append(
                                self.get_value_from_item(
                                    None, last_classifier_item[c], monit,
                                    view))
            if view.chart_type == MonitChartType.BAR:
                chart_data['timeline_data'].append("")
        return chart_data
        '''
            this.chart_data.timeline_data = ["Mo.", "Tu2"];
            this.chart_data.series_data = [
            {
                label: "SeriesA",
                data: [1, 2.2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
            },
            {
                label: "SeriesB",
                data: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
            },
            {
                label: "SeriesC",
                data: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
            },
            ];
        '''

    def get_monit_view_stats(self, monit, view) -> MonitViewStats:
        monit_key = self._get_key_from_monit(monit)
        general_view_items = self.MonitorViewItem.query.filter(
            self.MonitorViewItem.monit_key == monit_key,
            self.MonitorViewItem.classifier == None,
            self.MonitorViewItem.timestamp == None,
            self.MonitorViewItem.year == None,
            self.MonitorViewItem.month == None,
            self.MonitorViewItem.day == None,
            self.MonitorViewItem.hour == None).first()
        if general_view_items is not None:
            stats = MonitViewStats(
                min=general_view_items.values_min,
                max=general_view_items.values_max,
                average=general_view_items.values_average,
                count=general_view_items.values_count,
                sum=general_view_items.values_sum,
                last=general_view_items.values_last,
                last_event_at=general_view_items.last_event_at,
                last_reset_at=general_view_items.last_reset_at)
        else:
            stats = MonitViewStats(min=None,
                                   max=None,
                                   average=None,
                                   count=None,
                                   sum=None,
                                   last=None,
                                   last_event_at=None,
                                   last_reset_at=None)
        return stats

    def get_monit_view_classifier(self, monit, view) -> list:
        monit_key = self._get_key_from_monit(monit)
        monit_classifier_view_items = self.MonitorViewItem.query.filter(
            self.MonitorViewItem.monit_key == monit_key,
            self.MonitorViewItem.timestamp == None,
            self.MonitorViewItem.year == None,
            self.MonitorViewItem.month == None,
            self.MonitorViewItem.day == None,
            self.MonitorViewItem.hour == None).all()

        classifier = []
        if len(monit_classifier_view_items) == 0:
            pass
        else:
            if len(monit_classifier_view_items) == 1:
                classifier.append(monit_classifier_view_items[0].classifier)
            else:
                for c in monit_classifier_view_items:
                    if c.classifier is not None:
                        classifier.append(c.classifier)
        return classifier

    def register_monit(self, monit: Monit):
        self.monits[self._get_key_from_monit(monit)] = monit

    def raise_monitor_event(self, event: MonitorBaseEvent):
        logManager.info(f"Monitor event raised: {event}")
        e = self.MonitorEvent()
        e.name = event.name
        e.classifier = event.classifier
        e.value = event.value
        if event.timestamp is not None:
            e.timestamp = event.timestamp
        self.db.session.add(e)



from aurori.workspaces import Page
""" The log page
"""


class Users(Page):
    title = 'Users'  # Shown label of the page in the menu
    group = 'Admin'  # groupname multiple pages
    icon = 'supervisor_account'  # icon (in typeset of material design icons)
    route = '/admin/users'  # routing
    builder = 'frontend'  # page get build by the client (frontend)
    rank = 3.0  # ranks (double) the page higher values are at the top of the menu
    # groups will be ranked by the sum of the rank-values of their entries
    requireLogin = True  # login is required to view the page



from aurori import db


class OpenProject(db.Model):
    __tablename__ = 'openproject'
    id = db.Column(db.Integer,
                   primary_key=True,
                   index=True,
                   autoincrement=True,
                   unique=True)



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori.logs import logManager
from aurori.monits.types import MonitChartType, MonitType, MonitPropertyType
from aurori.monits.types import MonitTimeAggregationType, MonitTimeSpanDisplayType
from aurori.monits.views import MonitView, ALL


class Monit(object):
    group = None
    subgroup = None
    event = None
    description = ""
    monit_type = MonitType.PROPERTIES_ONLY
    monit_time_aggregation = MonitTimeAggregationType.HOURLY
    monit_display_time_span = MonitTimeSpanDisplayType.UNSET
    monit_display_property = MonitPropertyType.UNSET
    resetable = False  # allow the monit to be reseted mannualy
    floating_average_size = 100

    def __init__(self, workspace):
        self.workspace = workspace
        if not hasattr(self, 'name'):
            self.name = self.__class__.__name__
        self.views = []

    def add_barchart_view(self,
                          name,
                          label,
                          display_property,
                          display_classifier=ALL,
                          display_timespan=MonitTimeSpanDisplayType.WEEK,
                          fill_timeline_gaps=False,
                          description=""):
        view = MonitView(name,
                         label,
                         MonitChartType.BAR,
                         description,
                         display_classifier,
                         display_property,
                         display_timespan,
                         fill_timeline_gaps=fill_timeline_gaps)
        self_type = type(self)
        if self_type.monit_type == MonitType.PROPERTIES_ONLY:
            logManager.error(f"Unable to add bar chart monit view: \
                    {self.workspace.name}{str(self_type)}.{name} \
                        for a PROPERTIES_ONLY monit.")
            return

        if self_type.monit_type == MonitType.BUFFER:
            logManager.error(f"Unable to add bar chart monit view: \
                    {self.workspace.name}{str(self_type)}.{name} \
                        for a BUFFER type only scatter charts are allowed.")
            return
        self.views.append(view)

    def add_timelinechart_view(self,
                               name,
                               label,
                               display_property,
                               display_classifier=ALL,
                               display_timespan=MonitTimeSpanDisplayType.WEEK,
                               fill_timeline_gaps=False,
                               description=""):
        view = MonitView(name,
                         label,
                         MonitChartType.LINE,
                         description,
                         display_classifier,
                         display_property,
                         display_timespan,
                         fill_timeline_gaps=fill_timeline_gaps)
        self_type = type(self)
        if self_type.monit_type == MonitType.PROPERTIES_ONLY:
            logManager.error(f"Unable to add line chart monit view: \
                    {self.workspace.name}{str(self_type)}.{name} \
                        for a PROPERTIES_ONLY monit.")
            return

        if self_type.monit_type == MonitType.BUFFER:
            logManager.error(f"Unable to add line chart monit view: \
                    {self.workspace.name}{str(self_type)}.{name} \
                        for a BUFFER type only scatter charts are allowed.")
            return
        self.views.append(view)

    def add_bufferscatter_view(self,
                               name,
                               label,
                               display_property,
                               display_classifier=ALL,
                               description=""):
        self_type = type(self)
        view = MonitView(name,
                         label,
                         MonitChartType.SCATTER,
                         description,
                         display_classifier,
                         display_property,
                         fill_timeline_gaps=False)
        if self_type.monit_type == MonitType.PROPERTIES_ONLY:
            logManager.error(f"Unable to add scatter chart monit view: \
                    {self.workspace.name}{str(self_type)}.{name} \
                        for a PROPERTIES_ONLY monit.")
            return
        self.views.append(view)

    def add_property_view(self,
                          name,
                          label,
                          description="",
                          display_classifier=ALL):
        view = MonitView(name, label, MonitChartType.NONE, description,
                         display_classifier)
        self.views.append(view)

    def define_views(self):
        self_type = type(self)
        if self_type.monit_type == MonitType.PROPERTIES_ONLY:
            self.add_property_view("base", "Stats")
        self_type
        timespan = self_type.monit_display_time_span
        if timespan is MonitTimeSpanDisplayType.UNSET:
            timespan = MonitTimeSpanDisplayType.WEEK
        if self_type.monit_type == MonitType.TIMELINE:
            if self_type.monit_display_property is MonitPropertyType.UNSET:
                self.add_timelinechart_view("base", "Count",
                                            MonitPropertyType.COUNT, timespan)
            else:
                self.add_bufferscatter_view(
                    "base",
                    str(self_type.monit_display_property.value.title()),
                    self_type.monit_display_property)
        if self_type.monit_type == MonitType.BUFFER:
            if self_type.monit_display_property is MonitPropertyType.UNSET:
                self.add_bufferscatter_view("base", "Value",
                                            MonitPropertyType.LAST_VALUE)
            else:
                self.add_bufferscatter_view(
                    "base",
                    str(self_type.monit_display_property.value.title()),
                    self_type.monit_display_property)
        pass


"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

import copy
from aurori.common.objDict import ObjDict


class DataView(object):
    """Build up data-views
    """
    disable = False
    requireLogin = True
    requirePermission = None  # a permission is required in the meaning of one of the following

    def __init__(self, name=None, uri=None):
        self.description = 'UNKNOWN'
        if name is None:
            self.name = type(self).__name__
        else:
            self.name = name
        if uri is None:
            self.uri = self.name
        else:
            self.uri = uri
        self.requireLogin = True
        self.properties = []
        self.metadata = []
        self.dataAction = {}
        self.dataUpdateHandler = {}
        self.dataUpdates = []
        self.dataSyncs = []
        self.entrykey = None
        self.entrytype = None
        self.entryPropList = {}
        self.metaDataList = {}

    def createMeta(self):
        return ObjDict(self.metaDataList.copy())

    def addMailMeta(self, name, label="", group=None, description=""):
        meta = {'name': name, 'label': label, 'type': 'email'}
        self.metaDataList[name] = None
        self.metadata.append(meta)

    def addStringMeta(self, name, label="", group=None, description=""):
        meta = {'name': name, 'label': label, 'type': 'string'}
        self.metaDataList[name] = None
        self.metadata.append(meta)

    def addIntegerMeta(self, name, label="", group=None, description=""):
        meta = {'name': name, 'label': label, 'type': 'integer'}
        self.metaDataList[name] = None
        self.metadata.append(meta)

    def addDoubleMeta(self, name, label="", group=None, description=""):
        meta = {'name': name, 'label': label, 'type': 'double'}
        self.metaDataList[name] = None
        self.metadata.append(meta)

    def addBooleanMeta(self, name, label="", group=None, description=""):
        meta = {'name': name, 'label': label, 'type': 'boolean'}
        self.metaDataList[name] = None
        self.metadata.append(meta)

    def addTimeMeta(self, name, label="", group=None, description=""):
        meta = {'name': name, 'label': label, 'type': 'time'}
        self.metaDataList[name] = None
        self.metadata.append(meta)

    def addDateMeta(self, name, label="", group=None, description=""):
        meta = {'name': name, 'label': label, 'type': 'date'}
        self.metaDataList[name] = None
        self.metadata.append(meta)

    def addDatetimeMeta(self, name, label="", group=None, description=""):
        meta = {'name': name, 'label': label, 'type': 'datetime'}
        self.metaDataList[name] = None
        self.metadata.append(meta)

    def createEntry(self):
        return ObjDict(self.entryPropList.copy())

    def getProperties(self):
        properties = []
        for p in self.properties:
            pn = copy.copy(p)
            if pn['type'] == 'multiselect' or pn['type'] == 'select':
                try:
                    if callable(p['selection']):
                        pn['selection'] = p['selection']()
                    else:
                        pn['selection'] = p['selection']
                except Exception as e:
                    raise e
            properties.append(pn)
        return properties

    def addMailProperty(self,
                        name,
                        label="",
                        group=None,
                        updateHandler=None,
                        isKey=False,
                        readOnly=True,
                        description="",
                        hide=False,
                        sortable=True):
        prop = {'name': name, 'label': label, 'type': 'email'}
        if isKey == True:
            prop['isKey'] = True
            if self.entrykey != None:
                raise KeyError(
                    "DataView '{}' already have a key ({}) and cant be overridden with {}"
                    .format(self.name, self.entrykey, name))
            self.entrykey = name

        prop['readOnly'] = readOnly
        prop['group'] = group
        prop['hide'] = hide
        prop['sortable'] = sortable
        prop['description'] = description
        self.dataUpdateHandler[str(name)] = updateHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addStringProperty(self,
                          name,
                          label="",
                          group=None,
                          updateHandler=None,
                          isKey=False,
                          readOnly=True,
                          description="",
                          hide=False,
                          sortable=True):
        prop = {'name': name, 'label': label, 'type': 'string'}
        if isKey:
            prop['isKey'] = True
            if self.entrykey is not None:
                raise KeyError(
                    "DataView '{}' already have a key ({}) and cant be overridden with {}"
                    .format(self.name, self.entrykey, name))
            self.entrykey = name
        prop['readOnly'] = readOnly
        prop['group'] = group
        prop['hide'] = hide
        prop['sortable'] = sortable
        prop['description'] = description
        self.dataUpdateHandler[str(name)] = updateHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addDoubleProperty(self,
                          name,
                          label="",
                          group=None,
                          updateHandler=None,
                          isKey=False,
                          readOnly=True,
                          description="",
                          hide=False,
                          sortable=True):
        prop = {'name': name, 'label': label, 'type': 'double'}
        if isKey == True:
            prop['isKey'] = True
            if self.entrykey != None:
                raise KeyError(
                    "DataView '{}' already have a key ({}) and cant be overridden with {}"
                    .format(self.name, self.entrykey, name))
            self.entrykey = name
        prop['readOnly'] = readOnly
        prop['group'] = group
        prop['hide'] = hide
        prop['sortable'] = sortable
        prop['description'] = description
        self.dataUpdateHandler[str(name)] = updateHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addIntegerProperty(self,
                           name,
                           label="",
                           group=None,
                           updateHandler=None,
                           isKey=False,
                           readOnly=True,
                           description="",
                           hide=False,
                           sortable=True):
        prop = {'name': name, 'label': label, 'type': 'integer'}
        if isKey == True:
            prop['isKey'] = True
            if self.entrykey != None:
                raise KeyError(
                    "DataView '{}' already have a key ({}) and cant be overridden with {}"
                    .format(self.name, self.entrykey, name))
            self.entrykey = name
        prop['readOnly'] = readOnly
        prop['group'] = group
        prop['hide'] = hide
        prop['sortable'] = sortable
        prop['description'] = description
        self.dataUpdateHandler[str(name)] = updateHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addDatetimeProperty(self,
                            name,
                            label="",
                            group=None,
                            updateHandler=None,
                            isKey=False,
                            readOnly=True,
                            description="",
                            hide=False,
                            sortable=True):
        prop = {'name': name, 'label': label, 'type': 'datetime'}
        if isKey:
            prop['isKey'] = True
            if self.entrykey is not None:
                raise KeyError(
                    "DataView '{}' already have a key ({}) and cant be overridden with {}"
                    .format(self.name, self.entrykey, name))
            self.entrykey = name
        prop['readOnly'] = readOnly
        prop['group'] = group
        prop['hide'] = hide
        prop['sortable'] = sortable
        prop['description'] = description
        self.dataUpdateHandler[str(name)] = updateHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addTimeProperty(self,
                        name,
                        label="",
                        group=None,
                        updateHandler=None,
                        isKey=False,
                        readOnly=True,
                        description="",
                        hide=False,
                        sortable=True):
        prop = {'name': name, 'label': label, 'type': 'time'}
        if isKey == True:
            prop['isKey'] = True
            if self.entrykey != None:
                raise KeyError(
                    "DataView '{}' already have a key ({}) and cant be overridden with {}"
                    .format(self.name, self.entrykey, name))
            self.entrykey = name
        prop['readOnly'] = readOnly
        prop['group'] = group
        prop['hide'] = hide
        prop['sortable'] = sortable
        prop['description'] = description
        self.dataUpdateHandler[str(name)] = updateHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addDateProperty(self,
                        name,
                        label="",
                        group=None,
                        updateHandler=None,
                        isKey=False,
                        readOnly=True,
                        description="",
                        hide=False,
                        sortable=True):
        prop = {'name': name, 'label': label, 'type': 'date'}
        if isKey == True:
            prop['isKey'] = True
            if self.entrykey != None:
                raise KeyError(
                    "DataView '{}' already have a key ({}) and cant be overridden with {}"
                    .format(self.name, self.entrykey, name))
            self.entrykey = name
        prop['readOnly'] = readOnly
        prop['group'] = group
        prop['hide'] = hide
        prop['sortable'] = sortable
        prop['description'] = description
        self.dataUpdateHandler[str(name)] = updateHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addBooleanProperty(self,
                           name,
                           label="",
                           group=None,
                           updateHandler=None,
                           isKey=False,
                           readOnly=True,
                           description="",
                           hide=False,
                           sortable=True):
        prop = {'name': name, 'label': label, 'type': 'boolean'}
        if isKey == True:
            prop['isKey'] = True
            if self.entrykey != None:
                raise KeyError(
                    "DataView '{}' already have a key ({}) and cant be overridden with {}"
                    .format(self.name, self.entrykey, name))
            self.entrykey = name
        prop['readOnly'] = readOnly
        prop['group'] = group
        prop['hide'] = hide
        prop['sortable'] = sortable
        prop['description'] = description
        self.dataUpdateHandler[str(name)] = updateHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addSelectProperty(self,
                          name,
                          selectables,
                          label="",
                          group=None,
                          updateHandler=None,
                          isKey=False,
                          readOnly=True,
                          description="",
                          hide=False,
                          sortable=True):
        prop = {'name': name, 'label': label, 'type': 'select'}
        if isKey == True:
            prop['isKey'] = True
            if self.entrykey != None:
                raise KeyError(
                    "DataView '{}' already have a key ({}) and cant be overridden with {}"
                    .format(self.name, self.entrykey, name))
            self.entrykey = name
        prop['readOnly'] = readOnly
        prop['selection'] = selectables
        prop['group'] = group
        prop['hide'] = hide
        prop['sortable'] = sortable
        prop['description'] = description
        self.dataUpdateHandler[str(name)] = updateHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addMultiSelectProperty(self,
                               name,
                               selectables,
                               label="",
                               group=None,
                               updateHandler=None,
                               isKey=False,
                               readOnly=True,
                               description="",
                               hide=False,
                               sortable=True):
        prop = {'name': name, 'label': label, 'type': 'multiselect'}
        if isKey:
            prop['isKey'] = True
            if self.entrykey is not None:
                raise KeyError(
                    "DataView '{}' already have a key ({}) and cant be overridden with {}"
                    .format(self.name, self.entrykey, name))
            self.entrykey = name
        prop['readOnly'] = readOnly
        prop['selection'] = selectables
        prop['group'] = group
        prop['hide'] = hide
        prop['sortable'] = sortable
        prop['description'] = description
        self.dataUpdateHandler[str(name)] = updateHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addActionProperty(self,
                          name,
                          label,
                          action,
                          icon="",
                          actionHandler=None,
                          isKey=False,
                          readOnly=True,
                          color='rgb(165, 14, 45)',
                          description="",
                          hide=True):
        prop = {'name': name, 'label': label, 'type': 'action'}
        prop['isKey'] = False
        prop['icon'] = icon
        prop['color'] = color
        prop['action'] = action
        prop['hide'] = hide
        prop['description'] = description
        self.dataAction[str(action)] = actionHandler
        self.properties.append(prop)
        self.entryPropList[name] = None

    def addRemoveEntryOption(self, name, label):
        prop = {'name': name, 'label': label, 'type': 'remove'}
        prop['isKey'] = False
        prop['icon'] = 'delete'
        self.properties.append(prop)

    def emitUpdate(self, key, property, value):
        self.dataUpdates.append({
            'key': key,
            'property': property,
            'value': value,
            'view': self.uri
        })

    def emitSyncUpdate(self, key, view=None, workspace=None, query=None):
        if view is None:
            view = self.uri

        self.dataSyncs.append({
            'type': 'update',
            'key': key,
            'view': view,
            'workspace': workspace,
            'query': query
        })

    def emitSyncRemove(self, key, view=None, workspace=None, query=None):
        if view is None:
            view = self.uri

        self.dataSyncs.append({
            'type': 'remove',
            'key': key,
            'view': view,
            'workspace': workspace,
            'query': query
        })

    def emitSyncCreate(self, key, view=None, workspace=None, query=None):
        if view is None:
            view = self.name

        self.dataSyncs.append({
            'type': 'create',
            'key': key,
            'view': view,
            'workspace': workspace,
            'query': query
        })

    # Handler for getting the freshly build view
    def getViewHandler(self, user, workspace, query=None):
        raise NotImplementedError

    # Handler for getting the views meta-data
    def getViewMetaHandler(self, user, workspace):
        return {}

    # Handler for a request to create a new view entry
    def createViewEntryHandler(self, user, workspace):
        raise NotImplementedError

    # Handler for a request to remove a view entry
    def removeViewEntryHandler(self, user, workspace, key):
        raise NotImplementedError

    # Handler for a request to update a single view entry
    def updateViewEntryHandler(self, user, workspace, key, entry):
        raise NotImplementedError

    # Handler for view actions
    def executeViewActionHandler(self, user, workspace, action):
        return self.dataAction[action.viewAction](user, workspace, action,
                                                  action.entry[self.entrykey])

    def defineProperties(self):
        pass

    def defineMetadata(self):
        pass



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from datetime import datetime
from flask import Blueprint
from aurori.jobs.job import Job

jobs_bp = Blueprint('jobs', __name__)

from aurori.jobs.jobManager import JobManager
from aurori.logs import logManager
from inspect import isclass

jobManager = JobManager()


def trigger_job(job_key, args, user):
    if user is None:
        logManager.info("Internal job {} triggered".format(job_key))
    else:
        logManager.info("User {} triggered job {}".format(user.email, job_key))

    job_id = jobManager.run_job(user,
                                job_key,
                                args,
                                datetime.now(),
                                log_trigger=True)
    return job_id


def add_dated_job(user,
                  job,
                  args,
                  date=None,
                  workspace=None,
                  max_instances=10):
    if date is None:
        date = datetime.now()
    key = ""
    if workspace is not None:
        if type(workspace) == str:
            key += workspace + '/'
        elif isclass(workspace):
            logManager.error(
                "Class parameters are not allowed for add_dated_job")
            return
        else:
            key += workspace.name + '/'

    if type(job) == str:
        key += job
    elif isclass(workspace):
        logManager.error("Class parameters are not allowed for add_dated_job")
        return
    elif issubclass(type(job), Job):
        key += job.name
    else:
        logManager.error("Unknown type of job in add_dated_job")
        return
    jobManager.run_job(user, key, args, date, max_instances)


def print_job_list():
    actual_jobs = jobManager.scheduler.get_jobs()
    print(actual_jobs)
    all_jobs = jobManager.jobstore.get_all_jobs()
    print(all_jobs)


from aurori.jobs import routes



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from aurori import db
from sqlalchemy_utils import ArrowType
import arrow


class Message(db.Model):
    __tablename__ = 'messages'
    # nonvolatile data stored in the db
    id = db.Column(db.Integer, primary_key=True)
    recipient_email = db.Column(db.String, db.ForeignKey('users.email'))
    recipient = db.relationship("User",
                                backref=db.backref("messages", uselist=True))
    sender_name = db.Column(db.String(120), default="")
    subject = db.Column(db.String(120), default="")
    message_html = db.Column(db.UnicodeText(), default="")
    message_send_date = db.Column(ArrowType, default=arrow.utcnow)
    message_read = db.Column(db.Boolean, default=False)

    def __repr__(self):
        return '<Message from {} to {} with subject "{}" [{}] >'.format(
            self.sender_name,
            self.recipient_email,
            self.subject,
            self.message_send_date.strftime('%Y-%m-%d %H:%M:%S'),
        )



"""
The aurori project

Copyright (C) 2022  Marcus Drobisch,

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__authors__ = ["Marcus Drobisch"]
__contact__ = "aurori@fabba.space"
__credits__ = []
__license__ = "AGPLv3+"

from datetime import datetime, timedelta

from aurori.logs import logManager
from aurori.jobs.job import Job


class CleanupMonitorItemsJob(Job):

    cron = True
    hour = "*/4"
    minute = "0"
    disable = False
    description = "Cleanup monitor view items"

    def run(self, **kwargs):
        from aurori.monits import monit_manager
        from aurori.monits.models import MonitorEvent, MonitorViewItem
        now = datetime.utcnow()

        # remove all items from type buffer older than 1 week
        datetime_threshold = now.replace(minute=0, second=0,
                                         microsecond=0) - timedelta(days=7)
        monit_event_older_1week = MonitorViewItem.query.filter(
            MonitorViewItem.timestamp != None,
            MonitorViewItem.created_at < datetime_threshold).all()
        for m in monit_event_older_1week:
            self.db.session.delete(m)

        # remove all items from type timeline older than 2 month
        datetime_threshold = now.replace(minute=0, second=0,
                                         microsecond=0) - timedelta(days=60)
        monit_event_older_1week = MonitorViewItem.query.filter(
            MonitorViewItem.hour != None,
            MonitorViewItem.created_at < datetime_threshold).all()
        for m in monit_event_older_1week:
            self.db.session.delete(m)
        logManager.info("CleanupMonitorItemsJob done")